package com.glxp.udidl.admin.config;

import cn.hutool.core.util.StrUtil;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.dao.sys.SysUserMapper;
import com.glxp.udidl.admin.exception.JsonException;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

@Aspect
@Component
public class AuthAspect {

    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private GlobalConfig globalConfig;

    @Pointcut("@annotation(com.glxp.udidl.admin.annotation.AuthRuleAnnotation)")
    private void authMethod() {
    }

    @Before(value = "authMethod()")
    public void before(JoinPoint joinPoint) {
        if (globalConfig.isOpenAuth() == false)
            return;
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();

        if (attributes == null) {
            throw new JsonException(-1, "attributes = null");
        }
        HttpServletRequest request = attributes.getRequest();

        String admin_id = request.getHeader("ADMIN_ID");
        if (StrUtil.isNotBlank(admin_id) && "1".equals(admin_id)) {
            //管理员
            return;
        }

        String appId = request.getHeader("appId");
        String appSecret = request.getHeader("appSecret");
        if (StrUtil.isBlank(appId)) {
            throw new JsonException(401, "appId不能为空");
        }
        if (StrUtil.isBlank(appSecret)) {
            throw new JsonException(401, "appSecret不能为空");
        }
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        AuthRuleAnnotation action = method.getAnnotation(AuthRuleAnnotation.class);
        String perms = action.value();
        Integer res = sysUserMapper.hasAuthPerms(appId, appSecret, perms);
        if (res == 0)
            throw new JsonException(402, "无权限！");
    }
}
