package com.glxp.udidl.admin.controller.device;

import cn.hutool.core.util.StrUtil;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udid.Device;
import com.glxp.udidl.admin.entity.udid.DeviceEntity;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.req.udid.DeviceListRequest;
import com.glxp.udidl.admin.req.udid.DeviceSearchRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 未转换国家库查询与下载
 */
@Api(tags = "未转换国家库查询与下载接口")
@RestController
public class DeviceController {

    @Resource
    private DeviceService deviceService;


    @ApiOperation(value = "根据关键字查询医疗器械信息", response = DeviceEntity.class)
    @AuthRuleAnnotation("warehouse/device/search")
    @GetMapping("warehouse/device/search")
    public BaseResponse searchByKey(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();

        if (FilterUdiUtils.vailDiEmpty(key) != null) {
            return FilterUdiUtils.vailDiEmpty(key);
        }
        key = FilterUdiUtils.filterToDi(key);
        deviceListRequest.setKey(key);
        DeviceSearchRequest deviceSearchRequest = new DeviceSearchRequest();
        deviceSearchRequest.setZxxsdycpbs(key);
        deviceSearchRequest.setPage(1);
        deviceSearchRequest.setLimit(50);
        List<DeviceEntity> devices = deviceService.search(deviceSearchRequest);
        if (key == null || key.equals("") || devices == null || devices.size() == 0) {
            devices = new ArrayList<>();
            devices.add(new DeviceEntity());
        }
        long total = 1;
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        deviceEntityList.add(devices.get(devices.size() - 1));
        pageSimpleResponse.setList(deviceEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @ApiOperation(value = "根据DI查询医疗器械信息", response = DeviceEntity.class)
    @AuthRuleAnnotation("warehouse/device/searchbydi")
    @GetMapping("warehouse/device/searchbydi")
    public BaseResponse searchByDi(@Valid DeviceListRequest deviceListRequest, BindingResult bindingResult) {

        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        String key = deviceListRequest.getKey();

        if (FilterUdiUtils.vailDiEmpty(key) != null) {
            return FilterUdiUtils.vailDiEmpty(key);
        }
        Device device = deviceService.searchByZxxscpbs(key);
        long total = 1;
        DeviceEntity deviceEntity = new DeviceEntity();
        if (device != null)
            BeanUtils.copyProperties(device, deviceEntity);
        PageSimpleResponse<DeviceEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        List<DeviceEntity> deviceEntityList = new ArrayList<>();
        deviceEntityList.add(deviceEntity);
        pageSimpleResponse.setList(deviceEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }
}
