package com.glxp.udidl.admin.controller.device;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.info.FileInfoEntity;
import com.glxp.udidl.admin.req.ListPageRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.dataSync.ProductInfoDlService;
import com.glxp.udidl.admin.service.info.FileInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;

@Api(tags = "国家同步库文件信息接口")
@RestController
@RequestMapping("/file")
public class FileInfoController {

    @Autowired
    FileInfoService fileInfoService;
    @Autowired
    ProductInfoDlService productInfoDlService;

    @ApiOperation(value = "查询国家同步库同步文件列表", response = FileInfoEntity.class)
    @AuthRuleAnnotation("file/getlist")
    @PostMapping("/getlist")
    public BaseResponse getList(@RequestBody ListPageRequest param) {
        return fileInfoService.getList(param);
    }

    @ApiOperation(value = "下载国家同步库数据")
    @ApiImplicitParams({@ApiImplicitParam(name = "startDate", value = "开始时间"), @ApiImplicitParam(name = "endDate", value = "结束时间")})
    @AuthRuleAnnotation("file/exportToFile")
    @PostMapping("/exportToFile")
    public BaseResponse exportToFile(@DateTimeFormat(pattern = "yyyy-MM-dd") Date startDate, @DateTimeFormat(pattern = "yyyy-MM-dd") Date endDate) {
        return productInfoDlService.ExportToFile(startDate, endDate, "manual");
    }

    @ApiOperation(value = "下载文件")
    @ApiImplicitParam(name = "fileName", value = "文件名")
    @AuthRuleAnnotation("file/getFile")
    @GetMapping("/getFile")
    public void getFile(HttpServletResponse response, String fileName) {
        fileInfoService.getFile(response, fileName);
    }
}
