package com.glxp.udidl.admin.controller.device;

import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.res.PosTaggerResponse;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 赋码系统API
 */
@ApiIgnore
@RestController
public class PosTaggerController {

    @Resource
    ProductInfoService productInfoService;

    @AuthRuleAnnotation("udi/company/findByCreditNo")
    @GetMapping("udi/company/findByCreditNo")
    public BaseResponse findByCreditNo(ProductInfoFilterRequest productInfoFilterRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (StrUtil.isNotEmpty(productInfoFilterRequest.getTyshxydm())) {
            return ResultVOUtils.error(510, "通用社会信用代码不能为空");
        }
        List<ProductInfoEntity> productInfoEntities = productInfoService.filterUdiByCreditNo(productInfoFilterRequest);
        PageInfo<ProductInfoEntity> pageInfo;
        pageInfo = new PageInfo<>(productInfoEntities);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(productInfoEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("udi/company/findCoName")
    @GetMapping("udi/company/findCoName")
    public BaseResponse findCoName(ProductInfoFilterRequest productInfoFilterRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (productInfoFilterRequest.getTyshxydm() == null) {
            return ResultVOUtils.error(510, "通用社会信用代码不能为空");
        }
        List<String> names = new ArrayList<>();
        List<ProductInfoEntity> productInfoEntities = productInfoService.filterCpmctymc(productInfoFilterRequest);
        if (productInfoEntities != null && productInfoEntities.size() > 0) {

            for (ProductInfoEntity productInfoEntity : productInfoEntities) {
                names.add(productInfoEntity.getCpmctymc());
            }
        }
        PageInfo<String> pageInfo;
        pageInfo = new PageInfo<>(names);
        PageSimpleResponse<String> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(names);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @AuthRuleAnnotation("udi/company/findTreeData")
    @GetMapping("udi/company/findTreeData")
    public BaseResponse findTreeData(ProductInfoFilterRequest productInfoFilterRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (productInfoFilterRequest.getTyshxydm() == null || productInfoFilterRequest.getCpmctymc() == null) {
            return ResultVOUtils.error(510, "通用社会信用代码与产品用于名称不能为空");
        }
        List<ProductInfoEntity> productInfoEntities = productInfoService.findAllByUuid(productInfoFilterRequest);
        List<PosTaggerResponse> posTaggerResponses = new ArrayList<>();
        if (productInfoEntities != null && productInfoEntities.size() > 0) {
            posTaggerResponses = transferTreeData(productInfoEntities);
        }
        PageInfo<PosTaggerResponse> pageInfo;
        pageInfo = new PageInfo<>(posTaggerResponses);
        PageSimpleResponse<PosTaggerResponse> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(posTaggerResponses);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    public List<PosTaggerResponse> transferTreeData(List<ProductInfoEntity> productInfoEntities) {
        List<PosTaggerResponse> posTaggerResponses = new ArrayList<>();
        //产品名称按UUID分组
        Map<String, List<ProductInfoEntity>> cpmcMaps = productInfoEntities.stream().collect(Collectors.groupingBy(ProductInfoEntity::getUuid));
        for (List<ProductInfoEntity> uuidList : cpmcMaps.values()) {
            PosTaggerResponse ggxhBean = new PosTaggerResponse();
            List<ProductInfoEntity> ratioList = getPackRatio(uuidList);
            List<PosTaggerResponse.UdiInfoBean> udiInfoBeans = new ArrayList<>();
            for (ProductInfoEntity uuidEntity : ratioList) {

                BeanUtils.copyProperties(uuidEntity, ggxhBean);


                ggxhBean.setCphhhbh(uuidEntity.getCphhhbh());
                ggxhBean.setCplb(uuidEntity.getCplb());
                ggxhBean.setCpms(uuidEntity.getCpms());
                ggxhBean.setFlbm(uuidEntity.getFlbm());
                ggxhBean.setGgxh(uuidEntity.getGgxh());
                ggxhBean.setQxlb(uuidEntity.getQxlb());
                ggxhBean.setScbssfbhph(uuidEntity.getScbssfbhph());
                ggxhBean.setScbssfbhscrq(uuidEntity.getScbssfbhscrq());
                ggxhBean.setScbssfbhsxrq(uuidEntity.getScbssfbhsxrq());
                ggxhBean.setScbssfbhxlh(uuidEntity.getScbssfbhxlh());
                ggxhBean.setSpmc(uuidEntity.getSpmc());
                ggxhBean.setYbbm(uuidEntity.getYbbm());
                ggxhBean.setYlqxzcrbarmc(uuidEntity.getYlqxzcrbarmc());
                ggxhBean.setYlqxzcrbarywmc(uuidEntity.getYlqxzcrbarywmc());
                ggxhBean.setZczbhhzbapzbh(uuidEntity.getZczbhhzbapzbh());
                ggxhBean.setVersionNumber(uuidEntity.getVersionNumber());

                PosTaggerResponse.UdiInfoBean udiInfoBean = new PosTaggerResponse.UdiInfoBean();
                udiInfoBean.setBhxjcpbm(uuidEntity.getBhxjcpbm());
                udiInfoBean.setBhxjsl(uuidEntity.getBhxjsl());
                udiInfoBean.setBhzxxsbzsl(uuidEntity.getBhzxxsbzsl());
                if (uuidEntity.getDiType() == 1) {
                    udiInfoBean.setBzcj("主标识");
                } else if (uuidEntity.getDiType() == 2 || uuidEntity.getDiType() == 3) {
                    udiInfoBean.setBzcj("虚标识");
                } else {
                    udiInfoBean.setBzcj(uuidEntity.getBzcj());
                }

                udiInfoBean.setDiType(uuidEntity.getDiType());
                udiInfoBean.setNameCode(uuidEntity.getNameCode());
                udiInfoBean.setPackLevel(uuidEntity.getPackLevel());
                udiInfoBean.setPackRatio(uuidEntity.getPackRatio());
                udiInfoBean.setSjcpbm(uuidEntity.getSjcpbm());
                udiInfoBean.setZxxsbzbhsydysl(uuidEntity.getZxxsbzbhsydysl());
                udiInfoBeans.add(udiInfoBean);
            }
            ggxhBean.setUdiInfoBeans(udiInfoBeans);
            posTaggerResponses.add(ggxhBean);
        }
        return posTaggerResponses;
    }


    //包装比例
    public List<ProductInfoEntity> getPackRatio(List<ProductInfoEntity> uuidList) {
        if (uuidList != null && uuidList.size() > 0) {
            List<ProductInfoEntity> tops = getTop(uuidList);
            List<ProductInfoEntity> resultEntitys = new ArrayList<>();
            for (int i = 0; i < tops.size(); i++) {
                List<ProductInfoEntity> countEntitys = new ArrayList<>();
                ProductInfoEntity topEntity = tops.get(i);
                countEntitys.add(topEntity);
                List<Integer> nextArray = new ArrayList<>();
                nextArray.add(1);
                while (true) {
                    ProductInfoEntity nextEntity = getNext(topEntity, uuidList);
                    if (nextEntity != null) {
                        int currentCount = 1;
                        for (Integer count : nextArray) {
                            currentCount = currentCount * count;
                        }
                        currentCount = currentCount * topEntity.getBhxjsl();
                        nextArray.add(currentCount);
                        topEntity = nextEntity;
                        countEntitys.add(nextEntity);
                    } else {
                        break;
                    }
                }
                String packRatio = "";
                for (Integer ratio : nextArray) {
                    packRatio = packRatio + ":" + ratio;
                }
                packRatio = packRatio.substring(1);
                for (ProductInfoEntity countEntity : countEntitys) {
                    countEntity.setPackRatio(packRatio);
                }
                resultEntitys.addAll(countEntitys);
            }
            return resultEntitys;
        } else
            return null;
    }

    public ProductInfoEntity getNext(ProductInfoEntity current, List<ProductInfoEntity> uuidList) {
        for (int i = 0; i < uuidList.size(); i++) {
            ProductInfoEntity productInfoEntity = uuidList.get(i);
            ProductInfoEntity clone = new ProductInfoEntity();
            if (current.getBhxjcpbm().equals(productInfoEntity.getNameCode())) {
                BeanUtils.copyProperties(productInfoEntity, clone);
                return clone;
            }
        }
        return null;
    }

    public List<ProductInfoEntity> getTop(List<ProductInfoEntity> uuidList) {
        List<ProductInfoEntity> tops = new ArrayList<>();
        if (uuidList != null && uuidList.size() > 0) {
            for (int i = 0; i < uuidList.size(); i++) {
                ProductInfoEntity productInfoEntity = uuidList.get(i);
                if (productInfoEntity.getSjcpbm() == null || productInfoEntity.getSjcpbm().equals("")) {
                    tops.add(productInfoEntity);
                }
            }
        }
        return tops;
    }

    @AuthRuleAnnotation("udi/company/findByUuid")
    @GetMapping("udi/company/findByUuid")
    public BaseResponse findByDi(ProductInfoFilterRequest productInfoFilterRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        if (productInfoFilterRequest.getUuid() != null || productInfoFilterRequest.getNameCode() != null) {
            List<ProductInfoEntity> productInfoEntities = productInfoService.filterUdiByCreditNo(productInfoFilterRequest);
            PageInfo<ProductInfoEntity> pageInfo;
            pageInfo = new PageInfo<>(productInfoEntities);
            PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(productInfoEntities);
            return ResultVOUtils.success(pageSimpleResponse);
        } else
            return ResultVOUtils.error(510, "最小销售产品标识不能为空");
    }

}
