package com.glxp.udidl.admin.controller.device;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udi.UdiCompanyEntity;
import com.glxp.udidl.admin.entity.udid.UdiEntity;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.req.SyncCompanyReuqest;
import com.glxp.udidl.admin.req.SyncUdiRequest;
import com.glxp.udidl.admin.req.UdiCompanyRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.res.udid.ManuInfoResponse;
import com.glxp.udidl.admin.res.udid.ProductInfoResponse;
import com.glxp.udidl.admin.res.udid.VailUdiReponse;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.util.BeanUtils;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.admin.util.RedisUtil;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

//udi出入库管理系统API
@Api(tags = "产品信息接口")
@RestController
public class ProductInfoController {

    @Resource
    ProductInfoService productInfoService;
    @Resource
    UdiCompanyService udiCompanyService;
    @Resource
    RedisUtil redisUtil;

    //手持枪扫码查询
    @ApiOperation(value = "手持枪扫码查询", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/findBydi")
    @ApiImplicitParams({@ApiImplicitParam(name = "nameCode", value = "UDI码", required = true)})
    @GetMapping("udidl/device/findBydi")
    public BaseResponse findProductInfo(String nameCode) {
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        String key = nameCode;
        if (key != null) {
            key = FilterUdiUtils.getDiStr(key);
            productInfoFilterRequest.setNameCode(key);
            productInfoFilterRequest.setIsNewest(1);
        }
        if (StrUtil.isEmpty(productInfoFilterRequest.getNameCode())) {
            return ResultVOUtils.error(500, "UDI码不能为空");
        }
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        ProductInfoResponse productInfoResponse = new ProductInfoResponse();
        if (productInfoEntityList != null && productInfoEntityList.size() > 0) {
            ProductInfoEntity productInfoEntity = productInfoEntityList.get(productInfoEntityList.size() - 1);
            productInfoResponse.setProductInfoEntity(productInfoEntity);
            ProductInfoFilterRequest packFilter = new ProductInfoFilterRequest();
            packFilter.setUuid(productInfoEntity.getUuid());
            List<ProductInfoEntity> packInfos = productInfoService.findAll(packFilter);
            List<ProductInfoEntity> returns = new ArrayList<>();
            if (packInfos != null && packInfos.size() > 0) {
                for (ProductInfoEntity temp : packInfos) {
                    if (!temp.getNameCode().equals(productInfoEntity.getNameCode())) {
                        returns.add(temp);
                    }
                }
                productInfoResponse.setPackages(returns);
            }
            return ResultVOUtils.success(productInfoResponse);
        } else {
            return ResultVOUtils.error(500, "未找到产品信息");
        }
    }

    //管理平台扫码查询
    @ApiOperation(value = "管理平台扫码查询", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/wms/findBydi")
    @ApiImplicitParams({@ApiImplicitParam(name = "nameCode", value = "UDI码", required = true)})
    @GetMapping("udidl/wms/findBydi")
    public BaseResponse findBydiWms(String nameCode) {
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        String key = nameCode;
        if (FilterUdiUtils.vailDiEmpty(key) != null) {
            return FilterUdiUtils.vailDiEmpty(key);
        }
        UdiEntity udiEntity = FilterUdiUtils.getUdi(key);
        if (udiEntity == null) {
            return ResultVOUtils.error(500, "UDI码格式错误！");
        }
        productInfoFilterRequest.setIsNewest(1);
        productInfoFilterRequest.setNameCode(udiEntity.getUdi());
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        ProductInfoResponse productInfoResponse = new ProductInfoResponse();
        if (productInfoEntityList != null && productInfoEntityList.size() > 0) {
            ProductInfoEntity productInfoEntity = productInfoEntityList.get(productInfoEntityList.size() - 1);
            if (udiEntity != null) {
                productInfoEntity.setBatchNo(udiEntity.getBatchNo());
                productInfoEntity.setProduceDate(udiEntity.getProduceDate());
                productInfoEntity.setExpireDate(udiEntity.getExpireDate());
            }
            productInfoResponse.setProductInfoEntity(productInfoEntity);
            ProductInfoFilterRequest packFilter = new ProductInfoFilterRequest();
            packFilter.setUuid(productInfoEntity.getUuid());
            List<ProductInfoEntity> packInfos = productInfoService.findAll(packFilter);
            List<ProductInfoEntity> returns = new ArrayList<>();
            if (packInfos != null && packInfos.size() > 0) {
                for (ProductInfoEntity temp : packInfos) {
                    returns.add(temp);
                    if (udiEntity != null) {
                        productInfoEntity.setBatchNo(udiEntity.getBatchNo());
                        productInfoEntity.setProduceDate(udiEntity.getProduceDate());
                        productInfoEntity.setExpireDate(udiEntity.getExpireDate());
                    }
                }
                productInfoResponse.setPackages(returns);
            }
            return ResultVOUtils.success(productInfoResponse);
        } else {
            return ResultVOUtils.error(500, "未找到产品信息");
        }
    }

    //UDI自助平台/UDI管理系统/DI同步下载
    @ApiOperation(value = "UDI自助平台DI同步下载更新", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/udiwms/syncUdi")
    @GetMapping("udidl/udiwms/syncUdi")
    public BaseResponse syncUdi(SyncUdiRequest syncUdiRequest) {
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        BeanUtils.copyProperties(syncUdiRequest, productInfoFilterRequest);
        if (productInfoFilterRequest.getPage() == null || productInfoFilterRequest.getLimit() == null)
            return ResultVOUtils.error(500, "分页参数不能为空");
        List<ProductInfoEntity> productInfoEntityList = productInfoService.syncDlUdi(productInfoFilterRequest);
        return ResultVOUtils.success(productInfoEntityList);
    }

    //UDI自助平台/UDI管理系统/企业信息同步下载
    @ApiOperation(value = "UDI自助平台企业同步下载更新", response = UdiCompanyEntity.class)
    @AuthRuleAnnotation("udidl_company_all")
    @GetMapping("udidl/udiwms/syncCompany")
    public BaseResponse syncCompany(SyncCompanyReuqest syncCompanyReuqest) {
        UdiCompanyRequest udiCompanyRequest = new UdiCompanyRequest();
        BeanUtils.copyProperties(syncCompanyReuqest, udiCompanyRequest);
        if (udiCompanyRequest.getPage() == null || udiCompanyRequest.getLimit() == null)
            return ResultVOUtils.error(500, "分页参数不能为空");
        List<UdiCompanyEntity> udiCompanyEntities = udiCompanyService.syncDlCompany(udiCompanyRequest);
        return ResultVOUtils.success(udiCompanyEntities);
    }


    @ApiIgnore
    @ApiOperation(value = "根据主DI查询产品信息", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/filterUdi")
    @GetMapping("udidl/device/filterUdi")
    public BaseResponse filterUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        //过滤--1.参数不能全未空

        if (StrUtil.isEmpty(productInfoFilterRequest.getYlqxzcrbarmc()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getCpmctymc()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getNameCode()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getGgxh()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getUuid()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getZczbhhzbapzbh()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getDeviceRecordKey())) {
            return ResultVOUtils.error(500, "查询条件不足！");
        }
        if (StrUtil.isNotEmpty(productInfoFilterRequest.getNameCode()) && productInfoFilterRequest.getNameCode().length() <= 10) {
            return ResultVOUtils.error(500, "最小销售单元标识格式错误！");
        }

        if (StrUtil.isNotEmpty(productInfoFilterRequest.getYlqxzcrbarmc()) && productInfoFilterRequest.getYlqxzcrbarmc().length() <= 6) {
            return ResultVOUtils.error(500, "请输入完整的医疗器械注册备案人名称！");
        }
        if (StrUtil.isEmpty(productInfoFilterRequest.getYlqxzcrbarmc()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getCpmctymc()) &&
                StrUtil.isEmpty(productInfoFilterRequest.getNameCode())) {
            if (StrUtil.isNotEmpty(productInfoFilterRequest.getGgxh())) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
            if (StrUtil.isNotEmpty(productInfoFilterRequest.getZczbhhzbapzbh())) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
        }
        productInfoFilterRequest.setIsNewest(1);

        if (StrUtil.isEmpty(productInfoFilterRequest.getNameCode())) {
            productInfoFilterRequest.setDiType("1");
        }
        List<ProductInfoEntity> productInfoEntityList = productInfoService.filterUdi(productInfoFilterRequest);
        PageInfo<ProductInfoEntity> pageInfo;
        pageInfo = new PageInfo<>(productInfoEntityList);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(productInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @ApiOperation(value = "UDI自助平台DI查询接口", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/filterAllUdi")
    @GetMapping("udidl/device/filterAllUdi")
    public BaseResponse filterAllUdi(ProductInfoFilterRequest productInfoFilterRequest) {


        //过滤--1.参数不能全未空
        if ((productInfoFilterRequest.getYlqxzcrbarmc() == null || productInfoFilterRequest.getYlqxzcrbarmc().equals("")) &&
                (productInfoFilterRequest.getCpmctymc() == null || productInfoFilterRequest.getCpmctymc().equals("")) &&
                (productInfoFilterRequest.getNameCode() == null || productInfoFilterRequest.getNameCode().equals("")) &&
                (productInfoFilterRequest.getGgxh() == null || productInfoFilterRequest.getGgxh().equals("")) &&
                (productInfoFilterRequest.getUuid() == null || productInfoFilterRequest.getUuid().equals("")) &&
                (productInfoFilterRequest.getZczbhhzbapzbh() == null || productInfoFilterRequest.getZczbhhzbapzbh().equals("")) &&
                (productInfoFilterRequest.getDeviceRecordKey() == null || productInfoFilterRequest.getDeviceRecordKey().equals(""))
        ) {
            return ResultVOUtils.error(500, "参数不能为空！");
        }

        if (productInfoFilterRequest.getNameCode() != null && !productInfoFilterRequest.getNameCode().equals("") && productInfoFilterRequest.getNameCode().length() < 10) {
            return ResultVOUtils.error(500, "请输入完整的最小销售单元标识！");
        }

        if (productInfoFilterRequest.getYlqxzcrbarmc() != null && !productInfoFilterRequest.getYlqxzcrbarmc().equals("") && productInfoFilterRequest.getYlqxzcrbarmc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的医疗器械注册备案人名称！");
        }

        if ((productInfoFilterRequest.getYlqxzcrbarmc() == null || productInfoFilterRequest.getYlqxzcrbarmc().equals(""))
                && (productInfoFilterRequest.getNameCode() == null || productInfoFilterRequest.getNameCode().equals(""))
                && (productInfoFilterRequest.getCpmctymc() == null || productInfoFilterRequest.getCpmctymc().equals(""))

        ) {
            if (productInfoFilterRequest.getGgxh() != null && !productInfoFilterRequest.getGgxh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
            if (productInfoFilterRequest.getZczbhhzbapzbh() != null && !productInfoFilterRequest.getZczbhhzbapzbh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
        }

        if (productInfoFilterRequest.getCpmctymc() != null && !productInfoFilterRequest.getCpmctymc().equals("") && productInfoFilterRequest.getCpmctymc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的产品通用名称！");
        }
        productInfoFilterRequest.setIsNewest(1);


        String keyCache = productInfoFilterRequest.toCacheKey();
        if (StrUtil.isEmpty(keyCache)) {
            return ResultVOUtils.error(500, "查询条件不能为空！");
        }
        String totalStr = (String) redisUtil.get(keyCache);
        long total = 0;
        if (StrUtil.isEmpty(totalStr)) {
            total = productInfoService.filterUdiNoPageCount(productInfoFilterRequest);
            redisUtil.set(keyCache, total + "");
        } else {
            total = Long.parseLong(totalStr);
        }
        if (StrUtil.isEmpty(productInfoFilterRequest.getNameCode())) {
            productInfoFilterRequest.setDiType("1");
        }
        List<ProductInfoEntity> productInfoEntityList = productInfoService.filterUdiNoPage(productInfoFilterRequest);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        pageSimpleResponse.setList(productInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }


    @ApiOperation(value = "赋码系统下载全部产品信息接口", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/device/dlDiByPc")
    @GetMapping("udidl/device/dlDiByPc")
    public BaseResponse dlDiByPc(ProductInfoFilterRequest productInfoFilterRequest) {
        //过滤--1.参数不能全未空
        if ((productInfoFilterRequest.getYlqxzcrbarmc() == null || productInfoFilterRequest.getYlqxzcrbarmc().equals("")) &&
                (productInfoFilterRequest.getCpmctymc() == null || productInfoFilterRequest.getCpmctymc().equals("")) &&
                (productInfoFilterRequest.getNameCode() == null || productInfoFilterRequest.getNameCode().equals("")) &&
                (productInfoFilterRequest.getGgxh() == null || productInfoFilterRequest.getGgxh().equals("")) &&
                (productInfoFilterRequest.getUuid() == null || productInfoFilterRequest.getUuid().equals("")) &&
                (productInfoFilterRequest.getZczbhhzbapzbh() == null || productInfoFilterRequest.getZczbhhzbapzbh().equals("")) &&
                (productInfoFilterRequest.getDeviceRecordKey() == null || productInfoFilterRequest.getDeviceRecordKey().equals(""))
        ) {
            return ResultVOUtils.error(500, "参数不能为空！");
        }


        if (productInfoFilterRequest.getNameCode() != null && !productInfoFilterRequest.getNameCode().equals("") && productInfoFilterRequest.getNameCode().length() < 10) {
            return ResultVOUtils.error(500, "请输入完整的最小销售单元标识！");
        }

        if (productInfoFilterRequest.getYlqxzcrbarmc() != null && !productInfoFilterRequest.getYlqxzcrbarmc().equals("") && productInfoFilterRequest.getYlqxzcrbarmc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的医疗器械注册备案人名称！");
        }

        if (StrUtil.isEmpty(productInfoFilterRequest.getYlqxzcrbarmc()) && StrUtil.isEmpty(productInfoFilterRequest.getCpmctymc())
                && StrUtil.isEmpty(productInfoFilterRequest.getNameCode()) && StrUtil.isEmpty(productInfoFilterRequest.getTyshxydm())
        ) {
            if (productInfoFilterRequest.getGgxh() != null && !productInfoFilterRequest.getGgxh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
            if (productInfoFilterRequest.getZczbhhzbapzbh() != null && !productInfoFilterRequest.getZczbhhzbapzbh().equals("")) {
                return ResultVOUtils.error(500, "请输入医疗器械注册备案人名称");
            }
        }
        if (StrUtil.isNotEmpty(productInfoFilterRequest.getCpmctymc()) && productInfoFilterRequest.getCpmctymc().length() < 4) {
            return ResultVOUtils.error(500, "请输入完整的产品通用名称！");
        }
        productInfoFilterRequest.setIsNewest(1);

        String keyCache = productInfoFilterRequest.toCacheKey();
        if (StrUtil.isEmpty(keyCache)) {
            return ResultVOUtils.error(500, "查询条件不能为空！");
        }
        String totalStr = (String) redisUtil.get(keyCache);
        long total = 0;
        if (StrUtil.isEmpty(totalStr)) {
            total = productInfoService.filterUdiNoPageCount(productInfoFilterRequest);
            redisUtil.set(keyCache, total + "");
        } else {
            total = Long.parseLong(totalStr);
        }
        List<ProductInfoEntity> productInfoEntityList = productInfoService.filterUdiNoPage(productInfoFilterRequest);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(total);
        pageSimpleResponse.setList(productInfoEntityList);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @ApiOperation(value = "根据UUID查询产品信息", response = ProductInfoEntity.class)
    @ApiImplicitParams({@ApiImplicitParam(name = "uuid", value = "同一产品同一版本号唯一主键", required = true)})
    @AuthRuleAnnotation("udidl/device/filterByUuid")
    @GetMapping("udidl/device/filterByUuid")
    public BaseResponse filterByUuid(String uuid) {

        if (StrUtil.isEmpty(uuid)) {
            return ResultVOUtils.error(500, "查询条件不能为空！");
        }

        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        productInfoFilterRequest.setUuid(uuid);
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        return ResultVOUtils.success(productInfoEntityList);
    }

    @ApiOperation(value = "根据最小标识查询产品信息", response = ProductInfoEntity.class)
    @ApiImplicitParams({@ApiImplicitParam(name = "nameCode", value = "最小销售产品标识", required = true)})
    @AuthRuleAnnotation("udidl/device/filterByNameCode")
    @GetMapping("udidl/device/filterByNameCode")
    public BaseResponse filterByNameCode(String nameCode) {

        if (StrUtil.isEmpty(nameCode)) {
            return ResultVOUtils.error(500, "查询条件不能为空！");
        }
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        productInfoFilterRequest.setNameCode(nameCode);

        return ResultVOUtils.success(productInfoService.findLastVersion(productInfoFilterRequest));
    }


    @ApiOperation(value = "校验UDI完整性", response = ProductInfoEntity.class)
    @ApiImplicitParams({@ApiImplicitParam(name = "code", value = "UDI码", required = true)})
    @GetMapping("udidl/device/vailUdiCode")
    public BaseResponse vailUdiCode(String code) {

        if (StrUtil.isEmpty(code)) {
            return ResultVOUtils.error(500, "查询条件不能为空！");
        }
        UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
        if (udiEntity == null) {
            return ResultVOUtils.error(500, "UDI码格式错误！");
        } else {
            if (StrUtil.isNotEmpty(udiEntity.getUdi())) {
                ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
                productInfoFilterRequest.setNameCode(udiEntity.getUdi());
                List<ProductInfoEntity> productInfoEntityList = productInfoService.filterProductInfo(productInfoFilterRequest);
                if (CollUtil.isEmpty(productInfoEntityList)) {
                    return ResultVOUtils.error(500, "未查询到该UDI码国家库产品信息！");
                } else {
                    ProductInfoEntity productInfoEntity = productInfoEntityList.get(0);
                    String lostMsg = "";
                    boolean checkSuccess = true;
                    if ("是".equals(productInfoEntity.getScbssfbhph()) && StrUtil.isEmpty(udiEntity.getBatchNo())) {
                        checkSuccess = false;
                        lostMsg = lostMsg + ",批次号";
                    }
                    if ("是".equals(productInfoEntity.getScbssfbhscrq()) && StrUtil.isEmpty(udiEntity.getProduceDate())) {
                        checkSuccess = false;
                        lostMsg = lostMsg + ",生产日期";
                    }

                    if ("是".equals(productInfoEntity.getScbssfbhsxrq()) && StrUtil.isEmpty(udiEntity.getExpireDate())) {
                        checkSuccess = false;
                        lostMsg = lostMsg + ",失效日期";
                    }
                    if (("是".equals(productInfoEntity.getScbssfbhxlh()) && StrUtil.isEmpty(udiEntity.getSerialNo()))
                    ) {
                        checkSuccess = false;
                        lostMsg = lostMsg + ",序列号";
                    }
                    if (StrUtil.isNotEmpty(udiEntity.getSerialNo()) && udiEntity.getSerialNo().length() > 20) {
                        return ResultVOUtils.error(504, "序列号超出20位！");
                    }
                    if (StrUtil.isNotEmpty(udiEntity.getBatchNo()) && udiEntity.getBatchNo().length() > 20) {
                        return ResultVOUtils.error(504, "批次号超出20位！");
                    }
                    if (StrUtil.isEmpty(udiEntity.getSerialNo()) && StrUtil.isEmpty(udiEntity.getBatchNo())) {
                        checkSuccess = false;
                    }
                    if (checkSuccess) {
                        return ResultVOUtils.success(productInfoEntity);
                    } else {
                        return ResultVOUtils.error(500, "UDI码格式错误,缺少" + lostMsg.substring(1));
                    }
                }
            } else {
                return ResultVOUtils.error(500, "UDI码格式错误！");
            }
        }

    }


    @ApiOperation(value = "获取生产标识信息", response = ProductInfoEntity.class)
    @ApiImplicitParams({@ApiImplicitParam(name = "code", value = "UDI码", required = true)})
    @GetMapping("udidl/device/getManuInfo")
    public BaseResponse getManuInfo(String code) {

        if (StrUtil.isEmpty(code)) {
            return ResultVOUtils.error(500, "查询条件不能为空！");
        }
        UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
        if (udiEntity == null) {
            return ResultVOUtils.error(500, "UDI码格式错误！");
        } else {
            if (StrUtil.isNotEmpty(udiEntity.getUdi())) {
                ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
                productInfoFilterRequest.setNameCode(udiEntity.getUdi());
                List<ProductInfoEntity> productInfoEntityList = productInfoService.filterProductInfo(productInfoFilterRequest);
                if (CollUtil.isEmpty(productInfoEntityList)) {
                    return ResultVOUtils.error(500, "未查询到该UDI码国家库产品信息！");
                } else {
                    ProductInfoEntity productInfoEntity = productInfoEntityList.get(0);
                    ManuInfoResponse manuInfoResponse = new ManuInfoResponse();
                    BeanUtils.copyProperties(productInfoEntity, manuInfoResponse);
                    return ResultVOUtils.success(manuInfoResponse);
                }
            } else {
                return ResultVOUtils.error(500, "UDI码格式错误！");
            }
        }

    }
}
