package com.glxp.udidl.admin.controller.device;

import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udi.UdiCompanyEntity;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.req.FilterCompanyDiRequest;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.req.UdiCompanyRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.util.BeanUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;


//UDI查询APP API
@Api(tags = "APP查询企业信息接口")
@RestController
public class UdiCompanyController {

    @Resource
    UdiCompanyService udiCompanyService;
    @Resource
    ProductInfoService productInfoService;

    @ApiOperation(value = "查询企业信息列表", response = UdiCompanyEntity.class)
    @AuthRuleAnnotation("udi/company/filter")
    @GetMapping("udi/company/filter")
    public BaseResponse filterCompany(@Valid UdiCompanyRequest udiCompanyRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<UdiCompanyEntity> udiCompanyEntities = udiCompanyService.filterUdiCompany(udiCompanyRequest);
        return ResultVOUtils.success(udiCompanyEntities);
    }

    @ApiIgnore
    @AuthRuleAnnotation("udi/company/filtertByTD")
    @GetMapping("udi/company/filtertByTD")
    public BaseResponse filtertByTD(ProductInfoFilterRequest productInfoFilterRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<ProductInfoEntity> productInfoEntities = productInfoService.filterUdiByTyshxydm(productInfoFilterRequest);
        return ResultVOUtils.success(productInfoEntities);
    }

    @ApiOperation(value = "查询企业最新的产品信息", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udi/company/filtertUdi")
    @GetMapping("udi/company/filtertUdi")
    public BaseResponse filtertUdi(FilterCompanyDiRequest filterCompanyDiRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }

        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        BeanUtils.copyProperties(filterCompanyDiRequest, productInfoFilterRequest);
        if (StrUtil.isEmpty(productInfoFilterRequest.getTyshxydm())) {
            return ResultVOUtils.error(500, "统一社会信用代码不能为空！");
        }
        List<ProductInfoEntity> productInfoEntities;
        productInfoFilterRequest.setDiType("1");
        if (productInfoFilterRequest.isShowHis()) {
            productInfoEntities = productInfoService.filterUdiByTyshxydm(productInfoFilterRequest);
        } else {
            productInfoFilterRequest.setDiType("1");
            productInfoFilterRequest.setIsNewest(1);
            productInfoEntities = productInfoService.filterUdiByNewest(productInfoFilterRequest);
        }

        PageInfo<ProductInfoEntity> pageInfo;
        pageInfo = new PageInfo<>(productInfoEntities);
        PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(productInfoEntities);
        return ResultVOUtils.success(pageSimpleResponse);
    }
}
