package com.glxp.udidl.admin.controller.device;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.List;

//协调平台API
@ApiIgnore
@Api(tags = "协调平台接口")
@RestController
public class UdiCooperController {

    @Resource
    ProductInfoService productInfoService;

    @ApiOperation(value = "根据DI查询产品信息", response = ProductInfoEntity.class)
    @AuthRuleAnnotation("udidl/cooper/findBydi")
    @GetMapping("udidl/cooper/findBydi")
    public BaseResponse findProductInfo(ProductInfoFilterRequest productInfoFilterRequest) {

        String key = productInfoFilterRequest.getNameCode();
        if (key != null) {
            key = FilterUdiUtils.getDiStr(key);
            productInfoFilterRequest.setNameCode(key);
        } else {
            return ResultVOUtils.error(500, "产品条码不能为空！");
        }
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        if (productInfoEntityList != null && productInfoEntityList.size() > 0) {
            ProductInfoEntity productInfoEntity = productInfoEntityList.get(productInfoEntityList.size() - 1);
            return ResultVOUtils.success(productInfoEntity);
        } else {
            return ResultVOUtils.error(500, "未找到产品信息,请稍后重试");
        }
    }

}
