package com.glxp.udidl.admin.controller.device;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udid.*;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.req.DownloadUdiRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.inout.UdiDownloadService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

//外网全量下载api
@Api(tags = "外网全量下载医疗器械信息接口")
@RestController
public class UdiDownloadController {

    @Resource
    private UdiDownloadService udiDownloadService;

    @ApiOperation(value = "下载医疗器械信息", response = Device.class)
    @AuthRuleAnnotation("udi/download/device")
    @GetMapping("udi/download/device")
    public BaseResponse downloadDevice(@Valid DownloadUdiRequest downloadUdiRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Device> deviceList = udiDownloadService.downloadDevice(downloadUdiRequest);
        return ResultVOUtils.success(deviceList);
    }

    @ApiOperation(value = "下载医疗器械联系人信息", response = Contactlist.class)
    @AuthRuleAnnotation("udi/download/contactlist")
    @GetMapping("udi/download/contactlist")
    public BaseResponse downloadContactlist(@Valid DownloadUdiRequest downloadUdiRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Contactlist> deviceList = udiDownloadService.downloadContactlist(downloadUdiRequest);
        return ResultVOUtils.success(deviceList);
    }

    @ApiOperation(value = "下载临床器械信息", response = Deviceclinical.class)
    @AuthRuleAnnotation("udi/download/deviceclinical")
    @GetMapping("udi/download/deviceclinical")
    public BaseResponse downloadDeviceclinical(@Valid DownloadUdiRequest downloadUdiRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Deviceclinical> deviceList = udiDownloadService.downloadDeviceclinical(downloadUdiRequest);
        return ResultVOUtils.success(deviceList);
    }

    @ApiOperation(value = "下载医疗器械包装信息", response = Devicepackage.class)
    @AuthRuleAnnotation("udidl_udiDownload_all")
    @GetMapping("udi/download/devicepackage")
    public BaseResponse downloadDevicepackage(@Valid DownloadUdiRequest downloadUdiRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Devicepackage> deviceList = udiDownloadService.downloadDevicepackage(downloadUdiRequest);
        return ResultVOUtils.success(deviceList);
    }

    @ApiOperation(value = "下载医疗器械存储信息", response = Devicestorage.class)
    @AuthRuleAnnotation("udi/download/devicestorage")
    @GetMapping("udi/download/devicestorage")
    public BaseResponse downloadDevicestorage(@Valid DownloadUdiRequest downloadUdiRequest, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, bindingResult.getFieldError().getDefaultMessage());
        }
        List<Devicestorage> deviceList = udiDownloadService.downloadDevicestorage(downloadUdiRequest);
        return ResultVOUtils.success(deviceList);
    }

}
