package com.glxp.udidl.admin.controller.device;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udid.UdiEntity;
import com.glxp.udidl.admin.req.FilterUdiInfoRequest;
import com.glxp.udidl.admin.req.MutiScanRequest;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.res.UdiAppResponse;
import com.glxp.udidl.admin.service.dataSync.DeviceSyncService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RestController
public class UdiInfoController {

    @Resource
    ProductInfoService productInfoService;
    @Autowired
    DeviceSyncService deviceSyncService;


    @GetMapping("udi/basic/app/search")
    public BaseResponse filterRel(FilterUdiInfoRequest filterUdiRelRequest) {
        String nameCode;
        if (StrUtil.isEmpty(filterUdiRelRequest.getUdiCode())) {
            return ResultVOUtils.error(500, "UDI码不能为空！");
        }
        UdiEntity udiEntity = FilterUdiUtils.getUdi(filterUdiRelRequest.getUdiCode());
        if (udiEntity != null) {
            nameCode = udiEntity.getUdi();
        } else {
            return ResultVOUtils.error(500, "无效UDI码！");
        }
        if (StrUtil.isNotEmpty(nameCode)) {
            filterUdiRelRequest.setNameCode(nameCode);
        } else {
            filterUdiRelRequest.setDiType(1);
        }
        //查出UUID
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        productInfoFilterRequest.setNameCode(nameCode);
        productInfoFilterRequest.setIsNewest(1);
        List<ProductInfoEntity> productInfoEntityList = productInfoService.filterUdiByNewest(productInfoFilterRequest);
        if (CollUtil.isEmpty(productInfoEntityList)) {
            ThreadUtil.execAsync(() -> {
                deviceSyncService.searchDlByDi(productInfoFilterRequest.getNameCode());
            });
            return ResultVOUtils.error(500, "UDI码不存在，请检查后重试！");
        }
        List<ProductInfoEntity> results = productInfoService.selectByUuid(productInfoEntityList.get(0).getUuid());

        boolean checkSuccess = true;
        String lostMsg = "";
        if (udiEntity == null) {
            return ResultVOUtils.error(500, "UDI码格式错误！");
        } else {
            if (StrUtil.isNotEmpty(udiEntity.getUdi())) {
                ProductInfoEntity productInfoEntity = productInfoEntityList.get(0);
                if ("是".equals(productInfoEntity.getScbssfbhph()) && StrUtil.isEmpty(udiEntity.getBatchNo())) {
                    checkSuccess = false;
                    lostMsg = lostMsg + ",批次号";
                }
                if ("是".equals(productInfoEntity.getScbssfbhscrq()) && StrUtil.isEmpty(udiEntity.getProduceDate())) {
                    checkSuccess = false;
                    lostMsg = lostMsg + ",生产日期";
                }

                if ("是".equals(productInfoEntity.getScbssfbhsxrq()) && StrUtil.isEmpty(udiEntity.getExpireDate())) {
                    checkSuccess = false;
                    lostMsg = lostMsg + ",失效日期";
                }
                if (("是".equals(productInfoEntity.getScbssfbhxlh()) && StrUtil.isEmpty(udiEntity.getSerialNo()))
                ) {
                    checkSuccess = false;
                    lostMsg = lostMsg + ",序列号";
                }
                if (!checkSuccess) {
                    lostMsg = "UDI码格式错误,缺少" + lostMsg.substring(1);
                }
                if (StrUtil.isNotEmpty(udiEntity.getSerialNo()) && udiEntity.getSerialNo().length() > 20) {
                    checkSuccess = false;
                    lostMsg = lostMsg + ",序列号超出20位";
                }
                if (StrUtil.isNotEmpty(udiEntity.getBatchNo()) && udiEntity.getBatchNo().length() > 20) {
                    checkSuccess = false;
                    lostMsg = lostMsg + ",批次号超出20位";
                }
                if (StrUtil.isEmpty(udiEntity.getSerialNo()) && StrUtil.isEmpty(udiEntity.getBatchNo())) {
                    lostMsg = lostMsg + ",UDI码批次号与序列号不能全为空！";
                    checkSuccess = false;
                }
                if (checkSuccess) {
                    lostMsg = "校验成功！";
                } else {
                    lostMsg = lostMsg.substring(1);
                }
                for (ProductInfoEntity temp : results) {
                    temp.setBatchNo(udiEntity.getBatchNo());
                    temp.setExpireDate(udiEntity.getExpireDate());
                    temp.setProduceDate(udiEntity.getProduceDate());
                    temp.setSerialNo(udiEntity.getSerialNo());
                }
            }
        }
        UdiAppResponse udiAppResponse = new UdiAppResponse();
        udiAppResponse.setVailResult(checkSuccess);
        udiAppResponse.setVailMsg(lostMsg);
        udiAppResponse.setProductInfoEntities(results);
        return ResultVOUtils.success(udiAppResponse);
    }

    @GetMapping("udi/basic/app/mutilScan")
    public BaseResponse mutilScan(MutiScanRequest mutiScanRequest) {
        if (StrUtil.isEmpty(mutiScanRequest.getCode())) {
            return ResultVOUtils.error(500, "条码不能为空！");
        }
        String code = "";
        String originCode = mutiScanRequest.getOriginCode();
        if (originCode == null)
            originCode = "";
        code = originCode + mutiScanRequest.getCode();
        String newCode = mutiScanRequest.getCode();
        //重复判断
        if (StrUtil.isNotEmpty(mutiScanRequest.getOriginCode())) {
            String tempOriginCode = mutiScanRequest.getOriginCode();
            if (tempOriginCode.endsWith("\u001D")) {
                tempOriginCode = tempOriginCode.replace("\u001D", "");
            }
            UdiEntity originUdiEntity = FilterUdiUtils.getUdi(tempOriginCode);
            if (StrUtil.isNotEmpty(newCode)) {
                if (newCode.startsWith("10")) {
                    if (StrUtil.isNotEmpty(originUdiEntity.getBatchNo())) {
                        BaseResponse baseResponse = ResultVOUtils.error(501, "批次号重复添加");
                        baseResponse.setData(originCode);
                        return baseResponse;
                    } else {
                        String msg = checkNewCode(originUdiEntity, newCode);
                        if (msg != null) {
                            BaseResponse baseResponse = ResultVOUtils.error(501, msg);
                            baseResponse.setData(originCode);
                            return baseResponse;
                        }
                    }
                } else if (newCode.startsWith("17")) {
                    if (StrUtil.isNotEmpty(originUdiEntity.getExpireDate())) {
                        BaseResponse baseResponse = ResultVOUtils.error(501, "失效日期重复添加");
                        baseResponse.setData(originCode);
                        return baseResponse;
                    } else {
                        String msg = checkNewCode(originUdiEntity, newCode);
                        if (msg != null) {
                            BaseResponse baseResponse = ResultVOUtils.error(501, msg);
                            baseResponse.setData(originCode);
                            return baseResponse;
                        }
                    }
                } else if (newCode.startsWith("11")) {
                    if (StrUtil.isNotEmpty(originUdiEntity.getProduceDate())) {
                        BaseResponse baseResponse = ResultVOUtils.error(501, "生产日期重复添加");
                        baseResponse.setData(originCode);
                        return baseResponse;
                    } else {
                        String msg = checkNewCode(originUdiEntity, newCode);
                        if (msg != null) {
                            BaseResponse baseResponse = ResultVOUtils.error(501, msg);
                            baseResponse.setData(originCode);
                            return baseResponse;
                        }
                    }
                } else if (newCode.startsWith("21")) {
                    if (StrUtil.isNotEmpty(originUdiEntity.getSerialNo())) {
                        BaseResponse baseResponse = ResultVOUtils.error(501, "序列号重复添加");
                        baseResponse.setData(originCode);
                        return baseResponse;
                    } else {
                        String msg = checkNewCode(originUdiEntity, newCode);
                        if (msg != null) {
                            BaseResponse baseResponse = ResultVOUtils.error(501, msg);
                            baseResponse.setData(originCode);
                            return baseResponse;
                        }
                    }
                } else {
                    BaseResponse baseResponse = ResultVOUtils.error(501, "格式错误");
                    baseResponse.setData(originCode);
                    return baseResponse;
                }
            }
        }

        UdiEntity udiEntity = FilterUdiUtils.getUdi(code);
        if (udiEntity == null) {
            BaseResponse baseResponse = ResultVOUtils.error(501, "UDI码格式错误");
            baseResponse.setData(originCode);
            return baseResponse;
        }
        if (StrUtil.isNotEmpty(udiEntity.getBatchNo())) {
            if (code.endsWith("10" + udiEntity.getBatchNo())) {
                code = code + "\u001D";
            }
        }

        return ResultVOUtils.success(code);

    }


    public String checkNewCode(UdiEntity originEnity, String newCode) {
        UdiEntity newEntity = FilterUdiUtils.getUdi(originEnity.getUdi() + newCode);
        if (newEntity == null || originEnity == null) {
            return null;
        }
        if (StrUtil.isNotEmpty(originEnity.getBatchNo()) && StrUtil.isNotEmpty(newEntity.getBatchNo())) {
            return "批次号重复添加！";
        }
        if (StrUtil.isNotEmpty(originEnity.getProduceDate()) && StrUtil.isNotEmpty(newEntity.getProduceDate())) {
            return "生产日期重复添加！";
        }
        if (StrUtil.isNotEmpty(originEnity.getExpireDate()) && StrUtil.isNotEmpty(newEntity.getExpireDate())) {
            return "失效日期重复添加！";
        }
        if (StrUtil.isNotEmpty(originEnity.getSerialNo()) && StrUtil.isNotEmpty(newEntity.getSerialNo())) {
            return "序列号重复添加！";
        }
        return null;
    }


    //获取同步库UDI信息
    @AuthRuleAnnotation("")
    @GetMapping("udiwms/udiinfo/filterUdi")
    public BaseResponse filterUdi(FilterUdiInfoRequest filterUdiInfoRequest) {
        String nameCode = null;
        if (StrUtil.isNotEmpty(filterUdiInfoRequest.getCode())) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(filterUdiInfoRequest.getCode());
            if (udiEntity != null) {
                nameCode = udiEntity.getUdi();
            } else {
                return ResultVOUtils.error(500, "无效UDI码！");
            }
        } else
            nameCode = filterUdiInfoRequest.getNameCode();
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        BeanUtils.copyProperties(filterUdiInfoRequest, productInfoFilterRequest);
        if (StrUtil.isNotEmpty(nameCode)) {
            productInfoFilterRequest.setNameCode(nameCode);
        } else {
            productInfoFilterRequest.setDiType("1");
        }
        try {
            List<ProductInfoEntity> productInfoEntityList = productInfoService.filterUdi(productInfoFilterRequest);
            PageInfo<ProductInfoEntity> pageInfo;
            pageInfo = new PageInfo<>(productInfoEntityList);
            PageSimpleResponse<ProductInfoEntity> pageSimpleResponse = new PageSimpleResponse<>();
            pageSimpleResponse.setTotal(pageInfo.getTotal());
            pageSimpleResponse.setList(productInfoEntityList);
            return ResultVOUtils.success(pageSimpleResponse);

        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }


//    //校验UDI完整性
//    @AuthRuleAnnotation("")
//    @GetMapping("udidl/device/vailUdiCode")
//    public BaseResponse vailUdiCode(String udiCode) {
//
//        if (StrUtil.isEmpty(udiCode)) {
//            return ResultVOUtils.error(500, "查询条件不能为空！");
//        }
//        UdiEntity udiEntity = FilterUdiUtils.getUdi(udiCode);
//        if (udiEntity == null) {
//            return ResultVOUtils.error(500, "UDI码格式错误！");
//        } else {
//            if (StrUtil.isNotEmpty(udiEntity.getUdi())) {
//                ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
//                productInfoFilterRequest.setNameCode(udiEntity.getUdi());
//                List<ProductInfoEntity> productInfoEntityList = productInfoService.filterProductInfo(productInfoFilterRequest);
//                if (CollUtil.isEmpty(productInfoEntityList)) {
//                    return ResultVOUtils.error(500, "未查询到该UDI码国家库产品信息！");
//                } else {
//                    ProductInfoEntity productInfoEntity = productInfoEntityList.get(0);
//                    String lostMsg = "";
//                    boolean checkSuccess = true;
//                    if ("是".equals(productInfoEntity.getScbssfbhph()) && StrUtil.isEmpty(udiEntity.getBatchNo())) {
//                        checkSuccess = false;
//                        lostMsg = lostMsg + ",批次号";
//                    }
//                    if ("是".equals(productInfoEntity.getScbssfbhscrq()) && StrUtil.isEmpty(udiEntity.getProduceDate())) {
//                        checkSuccess = false;
//                        lostMsg = lostMsg + ",生产日期";
//                    }
//
//                    if ("是".equals(productInfoEntity.getScbssfbhsxrq()) && StrUtil.isEmpty(udiEntity.getExpireDate())) {
//                        checkSuccess = false;
//                        lostMsg = lostMsg + ",失效日期";
//                    }
//                    if (("是".equals(productInfoEntity.getScbssfbhxlh()) && StrUtil.isEmpty(udiEntity.getSerialNo()))
//                    ) {
//                        checkSuccess = false;
//                        lostMsg = lostMsg + ",序列号";
//                    }
//                    if (StrUtil.isNotEmpty(udiEntity.getSerialNo()) && udiEntity.getSerialNo().length() > 20) {
//                        return ResultVOUtils.error(504, "序列号超出20位！");
//                    }
//                    if (StrUtil.isNotEmpty(udiEntity.getBatchNo()) && udiEntity.getBatchNo().length() > 20) {
//                        return ResultVOUtils.error(504, "批次号超出20位！");
//                    }
//                    if (StrUtil.isEmpty(udiEntity.getSerialNo()) && StrUtil.isEmpty(udiEntity.getBatchNo())) {
//                        checkSuccess = false;
//                    }
//                    if (checkSuccess) {
//                        return ResultVOUtils.success(productInfoEntity);
//                    } else {
//                        return ResultVOUtils.error(500, "UDI码格式错误,缺少" + lostMsg.substring(1));
//                    }
//                }
//            } else {
//                return ResultVOUtils.error(500, "UDI码格式错误！");
//            }
//        }
//
//    }


    @AuthRuleAnnotation("")
    @GetMapping("udiwms/udiinfo/superSearch")
    public BaseResponse superSearch(FilterUdiInfoRequest filterUdiInfoRequest) {

        if (StrUtil.isEmpty(filterUdiInfoRequest.getNameCode()) && StrUtil.isEmpty(filterUdiInfoRequest.getCode())) {
            return ResultVOUtils.error(500, "DI不能为空！");
        }

        if (StrUtil.isNotEmpty(filterUdiInfoRequest.getCode())) {
            UdiEntity udiEntity = FilterUdiUtils.getUdi(filterUdiInfoRequest.getCode());
            if (udiEntity == null)
                return ResultVOUtils.error(500, "UDI码格式错误！");
            filterUdiInfoRequest.setNameCode(udiEntity.getUdi());
        }

        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        productInfoFilterRequest.setNameCode(filterUdiInfoRequest.getNameCode());
        productInfoFilterRequest.setPage(filterUdiInfoRequest.getPage());
        productInfoFilterRequest.setLimit(filterUdiInfoRequest.getLimit());
        try {

            return deviceSyncService.searchDlByDi(filterUdiInfoRequest.getNameCode());


        } catch (Exception e) {
            e.printStackTrace();
            return ResultVOUtils.error(500, "连接UDI数据下载服务出错！");
        }
    }


}
