package com.glxp.udidl.admin.controller.device;

import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.entity.udid.Contactlist;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.req.UpdateUdiRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.dataUpdate.DeviceUpdateService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 数据更新接口
 */
@RestController
public class UdiUpdateController {

    @Resource
    private DeviceUpdateService deviceUpdateService;

    @ApiOperation(value = "更新医疗器械信息接口", response = Contactlist.class)
    @AuthRuleAnnotation("udi/update/updateUdiData")
    @GetMapping("udi/update/updateUdiData")
    public BaseResponse updateData(UpdateUdiRequest updateUdiRequest) {
        if (updateUdiRequest.getType() == 0) {
            if (StrUtil.isBlank(updateUdiRequest.getStartDate()) || StrUtil.isBlank(updateUdiRequest.getEndDate())) {
                return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, "未指定更新时间");
            }
        } else if (updateUdiRequest.getType() == 1) {
            if (StrUtil.isBlank(updateUdiRequest.getNameCode())) {
                return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL, "未指定更新DI");
            }
        }

        ThreadUtil.execAsync(() -> {
            deviceUpdateService.updateUdiData(updateUdiRequest);
        });
        return ResultVOUtils.success();
    }


}
