package com.glxp.udidl.admin.controller.sys;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.dto.sys.SysUserModel;
import com.glxp.udidl.admin.dto.sys.SysUserParam;
import com.glxp.udidl.admin.enums.ResultEnum;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.sys.SysUserService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

@ApiIgnore
@RestController
@RequestMapping("sys/user")
public class SysUserController {

    @Autowired
    SysUserService sysUserService;

    @AuthRuleAnnotation("sys_user_all")
    @PostMapping("/list")
    public BaseResponse getList(@RequestBody SysUserParam param) {
        return sysUserService.list(param);
    }

    @AuthRuleAnnotation("sys_user_all")
    @PostMapping("/insert")
    public BaseResponse insert(@RequestBody SysUserModel model) {
        if (StrUtil.isBlank(model.getAppId()) || StrUtil.isBlank(model.getAppSecret())) {
            return ResultVOUtils.error(ResultEnum.PARAM_VERIFY_FALL);
        }
        return sysUserService.insert(model);
    }

    @AuthRuleAnnotation("sys_user_all")
    @PostMapping("update")
    public BaseResponse update(@RequestBody SysUserModel model) {
        return sysUserService.update(model);
    }

    @AuthRuleAnnotation("sys_user_all")
    @PostMapping("/detail")
    public BaseResponse detail(Integer id) {
        return sysUserService.detail(id);
    }

    @AuthRuleAnnotation("sys_user_all")
    @PostMapping("/delete")
    public BaseResponse delete(Integer id) {
        return sysUserService.delete(id);
    }

    @GetMapping("/generateSecret")
    public BaseResponse generateSecret() {
        return ResultVOUtils.success(IdUtil.fastSimpleUUID());
    }
}
