package com.glxp.udidl.admin.controller.udplat;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.dto.udplat.DeliveryGoods;
import com.glxp.udidl.admin.dto.udplat.HospitalGoods;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsParam;
import com.glxp.udidl.admin.entity.udplat.UdplatGoods;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.udplat.UdplatGoodsService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 目录清单
 */
@Api(tags = "阳光采购平台产品信息接口")
@RestController
@RequestMapping("udplat/goods")
public class UdplatGoodsController {

    @Autowired
    private UdplatGoodsService udplatGoodsService;

    @ApiOperation(value = "查询产品信息", response = UdplatGoods.class)
    @AuthRuleAnnotation("udplat/goods/list")
    @PostMapping("/list")
    public BaseResponse getList(@RequestBody UdplatGoodsParam param) {
        return udplatGoodsService.getList(param);
    }

    @ApiOperation(value = "Excel文件批量导入")
    @ApiImplicitParams({@ApiImplicitParam(name = "file", value = "excel文件"), @ApiImplicitParam(name = "type", value = "类型")})
    @AuthRuleAnnotation("udplat/goods/import")
    @PostMapping("/import")
    public BaseResponse importExcel(MultipartFile file, int type) {
        try {
            return udplatGoodsService.importExcel(file.getInputStream(), type);
        } catch (Exception e) {
            return ResultVOUtils.error(-1, e.getMessage());
        }
    }

    @ApiOperation(value = "导入产品信息")
    @AuthRuleAnnotation("udplat/goods/inportHospitalGoods")
    @PostMapping("/inportHospitalGoods")
    public BaseResponse importHospitalGoods(@RequestBody List<HospitalGoods> list) {
        return udplatGoodsService.importHospitalGoods(list);
    }

    @AuthRuleAnnotation("udplat/goods/importDeliveryGoods")
    @PostMapping("/importDeliveryGoods")
    public BaseResponse importDeliveryGoods(@RequestBody List<DeliveryGoods> list) {
        return udplatGoodsService.importDeliveryGoods(list);
    }

    @ApiOperation(value = "导出产品信息")
    @AuthRuleAnnotation("udplat/goods/export")
    @PostMapping("/export")
    public BaseResponse exportToFile(@RequestBody UdplatGoodsParam param) {
        return udplatGoodsService.exportToFile(param);
    }

    @ApiOperation(value = "查询单个产品信息", response = UdplatGoods.class)
    @ApiImplicitParam(name = "id", value = "产品id", required = true, dataType = "Integer")
    @AuthRuleAnnotation("udplat/goods/detail")
    @PostMapping("/detail")
    public BaseResponse detail(Integer id) {
        return udplatGoodsService.selectById(id);
    }

}
