package com.glxp.udidl.admin.controller.udplat;

import com.glxp.udidl.admin.annotation.AuthRuleAnnotation;
import com.glxp.udidl.admin.dto.device.ProductDetailModel;
import com.glxp.udidl.admin.dto.device.ProductGoodsMatchModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsMatchListModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsMatchModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsMatchParam;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.service.udplat.UdplatGoodsMatchService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "阳光采购平台产品对照接口")
@RestController
@RequestMapping("/udplat/goodsMatch")
public class UdplatGoodsMatchController {

    @Autowired
    private UdplatGoodsMatchService udplatGoodsMatchService;
    @Autowired
    private UdiCompanyService udiCompanyService;

    @ApiOperation(value = "保存对照信息")
    @AuthRuleAnnotation("udplat/goodsMatch/save")
    @PostMapping("/save")
    public BaseResponse save(@RequestBody UdplatGoodsMatchModel model) {
        return udplatGoodsMatchService.save(model);
    }

    @ApiOperation(value = "查询产品对照信息", response = UdplatGoodsMatchListModel.class)
    @AuthRuleAnnotation("udplat/goodsMatch/list")
    @PostMapping("/list")
    public BaseResponse getList(@RequestBody UdplatGoodsMatchParam param) {
        return udplatGoodsMatchService.getList(param);
    }

    @ApiOperation(value = "对照产品详细信息", response = ProductDetailModel.class)
    @ApiImplicitParam(name = "key", value = "关键字", required = true)
    @AuthRuleAnnotation("udplat/goodsMatch/productlist")
    @PostMapping("/productlist")
    public BaseResponse getProductList(String key) {
        return udplatGoodsMatchService.getProductList(key);
    }

    @ApiOperation(value = "查询产品详细信息", response = ProductGoodsMatchModel.class)
    @AuthRuleAnnotation("udplat/goodsMatch/productDetail")
    @PostMapping("/productDetail")
    public BaseResponse getProductInfoByUuid(String uuid) {
        return udplatGoodsMatchService.getProductInfoByUuid(uuid);
    }

    @ApiOperation(value = "查询医疗器械注册人名称")
    @ApiImplicitParam(name = "name", value = "医疗器械注册人名称，支持模糊查询", required = true, dataType = "String")
    @AuthRuleAnnotation("udplat/goodsMatch/searchByName")
    @PostMapping("/searchByName")
    public BaseResponse searchByName(String name) {
        return udiCompanyService.searchByName(name);
    }
}
