package com.glxp.udidl.admin.res.udid;

import lombok.Data;

import java.util.List;

@Data
public class DataSetResult {
    private int returnCode;         //交互码
    private String returnMsg;       //交互信息
    private int currentPageNumber;  //当前页码
    private int totalPageCount;     //总页数
    private int totalRecordCount;   //总记录数量
    private DataSet dataSet;

    @Data
    public static class DataSet {
        private List<DeviceInfo> deviceInfo;
        private List<ClinicalInfo> clinicalInfo;
        private List<StorageInfo> storageInfo;
        private List<PackingInfo> packingInfo;
    }

    /**
     * 设备信息
     */
    @Data
    public static class DeviceInfo {
        private String deviceHistoryKey;
        private String deviceRecordKey;
        private int versionNumber;
        private String versionStatus;
        private String versionTime;
        private String qtxxdwzlj;               //其他信息的网址链接
        private String scbssfbhsxrq;            //医疗器械生产标识是否包含失效日期：1 是 0 否
        private String scbssfbhscrq;            //医疗器械生产标识是否包含生产日期：1 是 0 否
        private String scbssfbhph;              //医疗器械生产标识是否包含批号：1 是 0 否
        private String scbssfbhxlh;             //医疗器械生产标识是否包含序列号：1 是 0 否
        private String tsccsm;                  //特殊使用尺寸说明
        private String tscchcztj;               //特殊存储或操作条件
        private String qxlb;
        private String cplb;
        private String yflbm;                   //原器械目录分类代码
        private String sfwwjbz;                 //医疗器械是否为已灭菌产品:1 是 0 否
        private String syqsfxyjxmj;             //医疗器械使用前是否需要进行灭菌:1 是 0 否
        private String cgzmraqxgxx;             //"磁共振（MR）安全相关信息；0 安全 ，1 条件安全， 2 不安全 ，3 说 明书或标签上面不包括 MR 安全信息
        private String ybbm;                    //27 位的医保耗材分类编码，多个之间用英文状态‘,’分隔
        private String sydycpbs;                //使用单元产品标识
        private String btcpbs;                  //本体标识医疗器械本体标识中的产品标识
        private String cpms;                    //产品其他描述信息
        private String zxxsdycpbs;              //最小销售单元产品标识
        private String flbm;                    //"器械目录分类代码
        private String cphhhbh;                 //产品货号或编号
        private String ylqxzcrbarmc;            //注册/备案人名称
        private String cpmctymc;                //产品名称/通用名称
        private String spmc;                    //商品名称
        private String ggxh;                    //规格型号
        private String sfybtzjbs;               //是否包含本体标识： 1 是 0 否
        private String sfwblztlcp;              //"医疗器械是否为包类产品/组套类产品；1 是 0 否
        private String zdcfsycs;                //医疗器械的最大重复使用次数
        private String mjfs;                    //"医疗器械的灭菌方式
        private String cpbsfbrq;                //标识发布时间；格式:2019-09-12
        private String tyshxydm;                //统一社会信用代码证号，境外企业填写境内代理人的统一社会信用代码
        private String cpbsbmtxmc;              //"产品标识编码体系名称，如 GS1，MA 码（IDcode）
        private String btcpbsyzxxsdycpbssfyz;   //是否与最小销售单元产品标识是否一致： 1 是 0 否
        private String sfbjwycxsy;              //标记为一次性使用:0 否， 1 是
        private String zczbhhzbapzbh;           //注册/备案证号，多个之间用英文状态‘,’分隔
        private String ylqxzcrbarywmc;          //注册/备案证对应的注册人/备案人的英文名称
        private String zxxsdyzsydydsl;          //最小销售单元中使用单元的数量
        private String tsrq;                    //医疗器械在流通领域停止销售的时间

        private String bszt;                    //"标识载体，1 一维码,2 二维码，3 RFID，4 其他；如存在多种，则按照以下 规则填写：1,2,3；‘,’为英文状态
        private String sfyzcbayz;               //是否与注册/备案标识一致：1 是 0 否
        private String zcbacpbs;                //注册/备案产品标识
        private String bssjzt;                  //"标识数据状态
        //文档有，接口返回无该字段
        private String cplx;                    //"产品类型：1 器械，2 体外诊断试剂
        private String hchzsb;                  //耗材或者设备:0 耗材， 1 设备
        private List<ContactInfo> contactList;
    }

    @Data
    public static class ContactInfo {
        private String qylxryx;     //企业联系人邮箱 1
        private String qylxrdh;     //企业联系人电话
        private String qylxrcz;     //企业联系人传真
    }

    @Data
    public static class ClinicalInfo {
        private String deviceHistoryKey;//历史key
        private String deviceRecordKey; // 数据库记录 key
        private String lcsycclx;    //尺寸类型
        private String ccz;         //尺寸值
        private String ccdw;        //计量单位
    }

    @Data
    public static class StorageInfo {
        private String deviceHistoryKey;//历史key
        private String deviceRecordKey; // 数据库记录 key
        private String cchcztj;         // 储存或操作条件
        private String zdz;             //最低值
        private String zgz;             //最高值
        private String jldw;        //计量单位
    }

    @Data
    public static class PackingInfo {
        private String deviceHistoryKey;//历史key
        private String deviceRecordKey; // 数据库记录 key
        private String bzcpbs;          //包装产品标识
        private String bznhxyjbzcpbs;   //包装内含下一级包装产品标识
        private String bznhxyjcpbssl;   //包装内含下一级产品标识数量
        private String cpbzjb;          //包装级别
    }
}
