package com.glxp.udidl.admin.service.dataSync;


import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.glxp.udidl.admin.constant.Constant;
import com.glxp.udidl.admin.entity.info.CompanyEntity;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udid.Device;
import com.glxp.udidl.admin.entity.udid.JobLog;
import com.glxp.udidl.admin.entity.udid.TokenEntity;
import com.glxp.udidl.admin.req.udid.DeviceRequest;
import com.glxp.udidl.admin.req.udid.DownloadDiRequest;
import com.glxp.udidl.admin.req.udid.DownloadHistoryRequest;
import com.glxp.udidl.admin.req.udid.TokenRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.udid.DataSetHistoryResult;
import com.glxp.udidl.admin.res.udid.DataSetResult;
import com.glxp.udidl.admin.res.udid.DataSetSingleResult;
import com.glxp.udidl.admin.res.udid.DataSetTotalResult;
import com.glxp.udidl.admin.service.info.CompanyService;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.HttpClient;
import com.glxp.udidl.admin.util.RedisUtil;
import com.glxp.udidl.admin.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 设备信息下载
 */
@Slf4j
@Service
public class DeviceDownloadService {

    private String token = "";

    private int reCount = 0;

    private CompanyEntity companyEntity;


    private String url = "sharing/get";

    @Resource
    private CompanyService companyService;
    @Resource
    private JobLogService jobLogService;                //日志
    @Resource
    private DeviceSaveService deviceSaveService;
    @Resource
    private DeviceService deviceService;
    @Resource
    private ProductInfoService productInfoService;
    @Resource
    private RedisUtil redisUtil;

    private void SetTokenEmpty() {
        token = "";
    }

    private static List<Map<String, String>> tokenList = new ArrayList<>();

    /**
     * 获取当前token
     *
     * @return
     */
    public String getToken() {
        token = redisUtil.get("UDISYNC_TOKEN");
        if (StrUtil.isEmpty(token)) {
            //查询下载数据使用的appid
            companyEntity = companyService.findByTaskType(Constant.TASK_TYPE_DOWNLOAD);
            TokenRequest tokenRequest = new TokenRequest();
            tokenRequest.setAppId(companyEntity.getAppId());
            tokenRequest.setAppSecret(companyEntity.getAppSecret());
            tokenRequest.setTyshxydm(companyEntity.getTyshxydm());
            String response = HttpClient.post("token/get", tokenRequest);
            TokenEntity tokenEntity = (TokenEntity) JSONObject.parseObject(response, TokenEntity.class);
            token = tokenEntity.getAccessToken();
            redisUtil.setEx("UDISYNC_TOKEN", token, 1 * 60 * 60 * 6);
        }
        return token;
    }

    public String getTokenNow(TokenRequest tokenRequest) {
        try {
            String response = HttpClient.post("token/get", tokenRequest);
            TokenEntity tokenEntity = (TokenEntity) JSONObject.parseObject(response, TokenEntity.class);
            if (tokenEntity.getReturnCode() == 1) {
                token = tokenEntity.getAccessToken();
                return "验证成功";
            } else {
                return "验证失败(国家库反馈信息:" + tokenEntity.getReturnMsg() + ")";
            }

        } catch (Exception e) {
            log.error("国家库网络服务器异常", e);
            return "国家库网络服务器异常!";
        }
    }


    /**
     * 获取产品标识详情数据
     *
     * @param deviceId
     * @return
     */
    public DataSetSingleResult downloadByDi(String deviceId) {
        DataSetSingleResult result = new DataSetSingleResult();
        DownloadDiRequest downloadDiRequest = new DownloadDiRequest();
        downloadDiRequest.setAccessToken(getToken());
        downloadDiRequest.setPrimaryDeviceId(deviceId);
        String response = "";
        try {
            response = HttpClient.post("sharing/single", downloadDiRequest);
        } catch (Exception e) {
            log.error("下载产品标识详情数据异常，响应结果：{}", response);
            result.setReturnCode(-1);
            result.setReturnMsg(e.getMessage());
            return result;//发生错误 退出
        }
        return JSONObject.parseObject(response, DataSetSingleResult.class);
    }

    /**
     * 获取单条数据
     *
     * @param rangeValue
     * @param requestType
     * @param page
     * @return
     */
    private DataSetResult getData(String rangeValue, int requestType, int page) {
        DataSetResult result = new DataSetResult();
        DeviceRequest deviceRequest = new DeviceRequest();
        deviceRequest.setAccessToken(getToken());
        deviceRequest.setRequestType(requestType + "");
        deviceRequest.setRangeValue(rangeValue);
        deviceRequest.setCurrentPageNumber(page + "");
        String response = "";
        try {
            Thread.sleep(3 * 1000);
            response = HttpClient.post(url, deviceRequest);
        } catch (InterruptedException e) {
            result.setReturnCode(-1);
            result.setReturnMsg(e.getMessage());
            return result;//发生错误 退出
        }
        return JSONObject.parseObject(response, DataSetResult.class);
    }

    /**
     * 获取单条数据
     *
     * @param page
     * @return
     */
    private synchronized DataSetResult getData(DeviceRequest deviceRequest, int page) {
        deviceRequest.setAccessToken(getToken());
        deviceRequest.setCurrentPageNumber(page + "");
        String response = "";

        String key = "downloadUdiData";
        if (redisUtil.getLock(key)) {
            log.info("获取到锁");
            redisUtil.setEx("downloadUdiData", "1", 5);
            //目前限制为5秒访问一次
            log.info("获取到锁的参数：" + page);
            response = HttpClient.post(url, deviceRequest);
            log.info("获取到结果，" + response);
        }
        log.info("开始解析结果");
        DataSetResult dataSetResult = JSONObject.parseObject(response, DataSetResult.class);
        log.info("结果解析结束");
        return dataSetResult;
    }


    /**
     * 获取数据
     *
     * @param rangeValue  "当按照天请求数据时，必填，输入参数按照 yyyy-MM-dd，当按照月请求数据时，必填，输入参数按照 yyyy-MM
     * @param requestType 请求范围，1 按天请求，2 按月请求
     * @return
     */
    public DataSetTotalResult getData(String rangeValue, int requestType) {
        DataSetTotalResult results = new DataSetTotalResult();
        int page = 1;
        int totalRecordCount = 0;
        boolean hasData = true;//是否还有数据，如无退出
        List<DataSetResult.DataSet> dataSets = new ArrayList<DataSetResult.DataSet>();
        while (hasData == true) {
            DataSetResult dataSetResult = getData(rangeValue, requestType, page);
            if (dataSetResult.getReturnCode() != 1 && reCount == 0) {
                token = "";
                reCount++;
                dataSetResult = getData(rangeValue, requestType, page);
            }
            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + JSON.toJSONString(dataSetResult) + ";";
                results.setMsg(msg);
                return results;//发生错误 退出
            } else {
                if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    dataSets.add(dataSetResult.getDataSet());
                    totalRecordCount = dataSetResult.getTotalRecordCount();
                } else { //获取不到数据，退出循环
                    hasData = false;
                    break;
                }
            }
            page++;
            totalRecordCount = dataSetResult.getTotalRecordCount();
        }
        results.setData(dataSets);
        results.setTotalRecordCount(totalRecordCount);
        reCount = 0;
        return results;
    }

    /**
     * 按天/按月下载
     *
     * @param ：如果是天，就是单日时间，如果是月份，就是一个月份 yyyy-MM
     * @return
     */
    public DataSetTotalResult getData(DeviceRequest deviceRequest) {
        DataSetTotalResult results = new DataSetTotalResult();
        int page = 1;
        int totalRecordCount = 0;
        boolean hasData = true; //是否还有数据，如无退出
        List<DataSetResult.DataSet> dataSets = new ArrayList<>();
        while (hasData) {
            DataSetResult dataSetResult = getData(deviceRequest, page);
            if (dataSetResult.getReturnCode() != 1 && reCount == 0) {
                token = "";
                reCount++;
                dataSetResult = getData(deviceRequest, page);
            }

            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + JSON.toJSONString(dataSetResult) + ";";
                results.setMsg(msg);
                return results;
            } else {
                if (null != dataSetResult.getDataSet()) {
                    if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                        dataSets.add(dataSetResult.getDataSet());
                        dataSetResult.getTotalRecordCount();
                    } else {
                        hasData = false;
                        break;
                    }
                } else {
                    hasData = false;
                    break;
                }
            }
            page++;
            totalRecordCount = dataSetResult.getTotalRecordCount();
        }
        results.setData(dataSets);
        results.setTotalRecordCount(totalRecordCount);
        reCount = 0;
        return results;
    }


    public boolean getDataAndSave(DeviceRequest deviceRequest, JobLog jobLog, Integer dlType) {
        DataSetTotalResult results = new DataSetTotalResult();
        int page = 1;
        if (StrUtil.isNotEmpty(deviceRequest.getCurrentPageNumber())) {
            page = Integer.parseInt(deviceRequest.getCurrentPageNumber());
        }
        boolean hasData = true; //是否还有数据，如无退出
        while (hasData) {
            redisUtil.set("allUpdatePage", page+"");
            DataSetResult dataSetResult = getData(deviceRequest, page);
            if (dataSetResult.getReturnCode() != 1 && reCount == 0) {
                token = "";
                reCount++;
                try {
                    dataSetResult = getData(deviceRequest, page);
                } catch (Exception e) {
                    jobLog.setStatus(1);
                    jobLog.setType(Constant.LOG_TYPE_ERROR);
                    jobLog.setMsg("下载失败：" + e.getMessage());
                    jobLog.setLastUploadRequest(JSONUtil.toJsonStr(deviceRequest));
                    jobLogService.update(jobLog);
                    return false;
                }
            }
            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + JSON.toJSONString(dataSetResult) + ";";
                results.setMsg(msg);
                jobLog.setStatus(1);
                jobLog.setMsg("下载失败：" + msg);
                jobLog.setType(Constant.LOG_TYPE_ERROR);
                jobLog.setStatus(1);
                jobLog.setLastUploadRequest(JSONUtil.toJsonStr(deviceRequest));
                jobLogService.update(jobLog);
                return false;
            } else {
                if (null != dataSetResult.getDataSet()) {
                    if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                        List<DataSetResult.DataSet> dataSets = new ArrayList<>();
                        dataSets.add(dataSetResult.getDataSet());
                        try {
                            int insertCount = deviceSaveService.DeviceSave(dataSets, jobLog.getDownloadDate(), dlType);
                            jobLog.setRealCount(jobLog.getRealCount() + dataSetResult.getDataSet().getDeviceInfo().size());
                            jobLog.setInsertCount(jobLog.getInsertCount() + insertCount);
                            jobLog.setTotalCount(dataSetResult.getTotalRecordCount());
                            jobLog.setMsg("总条数：" + dataSetResult.getTotalRecordCount() + ";下载条数：" + jobLog.getRealCount() + ";插入条数：" + jobLog.getInsertCount());
                            jobLog.setLastUploadRequest(JSONUtil.toJsonStr(deviceRequest));
                            jobLogService.update(jobLog);
                        } catch (Exception e) {
                            log.error("插入数据失败", e);
                            jobLog.setStatus(1);
                            jobLog.setType(Constant.LOG_TYPE_ERROR);
                            jobLog.setMsg("数据写入失败" + e.getMessage());
                            jobLog.setUpdateTime(new Date());
                            jobLog.setInsertCount(0);
                            jobLog.setLastUploadRequest(JSONUtil.toJsonStr(deviceRequest));
                            jobLogService.update(jobLog);
                            return false;
                        }
                    } else {
                        hasData = false;
                        break;
                    }
                } else {
                    hasData = false;
                    break;
                }
            }
            page++;

        }
        return true;
    }


    public List<String> getKeyList(DataSetResult.DataSet ds) {
        return ds.getDeviceInfo().stream().map(item -> item.getDeviceRecordKey()).collect(Collectors.toList());
    }

    public DataSetTotalResult getHistoryData2(List<String> deviceRecordKeys) {
        DataSetTotalResult result = new DataSetTotalResult();
        int limit = 10;
        int totalPage = (deviceRecordKeys.size() - 1) / limit + 1;
        int page = 1;
        while (page <= totalPage) {
            List<String> list = deviceRecordKeys.stream().skip((page - 1) * limit).limit(limit).collect(Collectors.toList());
            DataSetTotalResult res = getHistoryData1(list);
            if (res.getCode() != 1)
                return res;
            if (res.getData() != null && res.getData().size() > 0) {
                if (res.getTotalRecordCount() != res.realRecordCount())
                    log.info("行数不一致：" + res.getTotalRecordCount() + "," + res.realRecordCount());
                result.setTotalRecordCount(result.getTotalRecordCount() + res.getTotalRecordCount());
                List<DataSetResult.DataSet> dataList = result.getData();
                dataList.addAll(res.getData());
                result.setData(dataList);
            }
            page++;
        }
        return result;
    }

    /**
     * 获取历史数据
     *
     * @param deviceRecordKeys
     * @return
     */
    public DataSetTotalResult getHistoryData1(List<String> deviceRecordKeys) {
        DataSetTotalResult results = new DataSetTotalResult();
        String historyUrl = "sharing/history/get";
        int page = 1;
        int totalRecordCount = 0;
        DownloadHistoryRequest deviceHistoryRequest = new DownloadHistoryRequest();
        deviceHistoryRequest.setAccessToken(getToken());
        List<DownloadHistoryRequest.DataSetBean> dataSetBeans = new ArrayList<>();
        deviceRecordKeys.forEach(item -> {
            DownloadHistoryRequest.DataSetBean keys = new DownloadHistoryRequest.DataSetBean();
            keys.setDeviceRecordKey(item);
            dataSetBeans.add(keys);
        });
        deviceHistoryRequest.setDataSet(dataSetBeans);
        List<DataSetResult.DataSet> dataSets = new ArrayList<DataSetResult.DataSet>();
        boolean hasData = true;
        while (hasData == true) {
            deviceHistoryRequest.setCurrentPageNumber(page + "");
            String response = "";
            try {
                Thread.sleep(3 * 1000);
                response = HttpClient.post(historyUrl, deviceHistoryRequest);
            } catch (InterruptedException e) {
                log.error(e.getMessage(), e);
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + e.getMessage() + ";";
                results.setMsg(msg);
                return results;//发生错误 退出
            }
            DataSetHistoryResult dataSetResult = JSONObject.parseObject(response, DataSetHistoryResult.class);
            if (dataSetResult.getReturnCode() != 1 && reCount == 0) {//历史记录失败后重新下载1次
                token = "";
                reCount++;
                continue;
            }
            page++;
            if (dataSetResult.getReturnCode() != 1) {
                results.setCode(-1);
                String msg = results.getMsg();
                msg = msg + dataSetResult.getReturnMsg();
                results.setMsg(msg);
                return results;//发生错误 退出
            } else {
                if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    dataSets.add(dataSetResult.getDataSet());
                    totalRecordCount = totalRecordCount + dataSetResult.getTotalRecordCount();
                } else {
                    hasData = false;
                    break;
                }
            }


        }
        reCount = 0;
        results.setData(dataSets);
        results.setTotalRecordCount(totalRecordCount);
        return results;
    }

    public DataSetTotalResult getDataAll(int days) {
        DataSetTotalResult result = new DataSetTotalResult();
        for (int i = 1; i <= days; i++) {
            String day = DateUtil.getLastDay(-i);
            DataSetTotalResult res = getData(day, 1);
            if (res.getCode() != 1) {
                result.setCode(res.getCode());
                result.setMsg(res.getMsg());
                return result;
            }
            int count = result.getTotalRecordCount();
            count = count + res.getTotalRecordCount();
            result.setTotalRecordCount(count);
            List<DataSetResult.DataSet> data = result.getData();
            if (data == null)
                data = res.getData();
            else
                data.addAll(res.getData());
            log.info("data数量：" + data.size());
            result.setData(data);
        }

        List<String> keyList = new ArrayList<String>();
        for (DataSetResult.DataSet ds : result.getData()) {
            List<DataSetResult.DeviceInfo> list = ds.getDeviceInfo().stream().filter(d -> d.getVersionNumber() > 1).collect(Collectors.toList());
            if (list != null && list.size() > 0) {
                for (DataSetResult.DeviceInfo deviceInfo : list)
                    keyList.add(deviceInfo.getDeviceRecordKey());
            }
        }
        if (keyList.size() > 0) {
            DataSetTotalResult res = getHistoryData1(keyList);
            if (res.getCode() == 1) {
                if (res.getData().size() > 0) {
                    int count = result.getTotalRecordCount();
                    count = count + res.getTotalRecordCount();
                    result.setTotalRecordCount(count);
                    List<DataSetResult.DataSet> data = result.getData();
                    data.addAll(res.getData());
                    result.setData(data);
                }
            } else {
                JobLog jobLog = new JobLog();
                jobLog.setType(Constant.LOG_TYPE_ERROR);
                jobLog.setMsg(res.getMsg());
                jobLog.setCreateTime(new Date());
                jobLogService.insert(jobLog);
            }

        }
        return result;
    }

    //按月份下载 边下载边保存
    public BaseResponse downloadByMonth(String month, int page) {
        try {
            int totalPage = page;
            int totalRecordCount = 0;
            while (page <= totalPage) {
                DataSetResult dataSetResult = getData(month, 2, page);
                if (dataSetResult.getReturnCode() != 1) {
                    String msg = "下载到第" + page + "页发生错误";
                    //发生错误，存储日志
                    JobLog jobLog = new JobLog();
                    jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
                    jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
                    jobLog.setType(Constant.LOG_TYPE_ERROR);
                    jobLog.setMsg(msg);
                    jobLog.setContent(dataSetResult.getReturnMsg());
                    jobLog.setCreateTime(new Date());
                    jobLogService.insert(jobLog);
                    return ResultVOUtils.error(-1, msg + ",错误信息：" + dataSetResult.getReturnMsg());
                } else if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    //存储数据
                    List<DataSetResult.DataSet> dataSets = new ArrayList<>();
                    dataSets.add(dataSetResult.getDataSet());
                    deviceSaveService.DeviceSave(dataSets, null, null);
                }
                totalPage = dataSetResult.getTotalPageCount();
                totalRecordCount = dataSetResult.getTotalRecordCount();
                page++;
            }
            JobLog jobLog = new JobLog();
            jobLog.setType(Constant.LOG_TYPE_INFO);
            jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
            jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
            jobLog.setMsg("月份：" + month + ",执行成功！");
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
            jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
            jobLog.setType(Constant.LOG_TYPE_ERROR);
            jobLog.setMsg("月份：" + month + ",执行失败:" + e.getMessage());
            jobLog.setContent(JSON.toJSONString(e.getStackTrace()));
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
            return ResultVOUtils.error(-1, jobLog.getMsg());
        }
        return ResultVOUtils.success();
    }

    //按月份下载 边下载边保存
    public BaseResponse downloadByMonthTest(String month, int page) {
        try {
            int totalPage = page;
            int totalRecordCount = 0;
            while (page <= totalPage) {
                DataSetResult dataSetResult = getData(month, 2, page);
                if (dataSetResult.getReturnCode() != 1) {
                    String msg = "下载到第" + page + "页发生错误";
                    //发生错误，存储日志
                    JobLog jobLog = new JobLog();
                    jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
                    jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
                    jobLog.setType(Constant.LOG_TYPE_ERROR);
                    jobLog.setMsg(msg);
                    jobLog.setContent(dataSetResult.getReturnMsg());
                    jobLog.setCreateTime(new Date());
                    jobLogService.insert(jobLog);
                    return ResultVOUtils.error(-1, msg + ",错误信息：" + dataSetResult.getReturnMsg());
                } else if (dataSetResult.getDataSet().getDeviceInfo() != null && dataSetResult.getDataSet().getDeviceInfo().size() > 0) {
                    //存储数据
                    List<DataSetResult.DataSet> dataSets = new ArrayList<>();
                    dataSets.add(dataSetResult.getDataSet());
                    for (DataSetResult.DataSet ds : dataSets) {
                        for (DataSetResult.DeviceInfo item : ds.getDeviceInfo()) {
                            Device device = new Device();
                            device.setHchzsb(item.getHchzsb());
                            device.setCplx(item.getCplx());
                            device.setDevicerecordkey(item.getDeviceRecordKey());
                            deviceService.updateCplx(device);

                            ProductInfoEntity productInfoEntity = new ProductInfoEntity();
                            productInfoEntity.setHchzsb(item.getHchzsb());
                            productInfoEntity.setCplx(item.getCplx());
                            productInfoEntity.setDeviceRecordKey(item.getDeviceRecordKey());
                            productInfoService.updateCplx(productInfoEntity);

                        }
                    }
                }
                totalPage = dataSetResult.getTotalPageCount();
                totalRecordCount = dataSetResult.getTotalRecordCount();
                page++;
            }
            JobLog jobLog = new JobLog();
            jobLog.setType(Constant.LOG_TYPE_INFO);
            jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
            jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
            jobLog.setMsg("月份：" + month + ",执行成功！");
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
            jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
            jobLog.setType(Constant.LOG_TYPE_ERROR);
            jobLog.setMsg("月份：" + month + ",执行失败:" + e.getMessage());
            jobLog.setContent(JSON.toJSONString(e.getStackTrace()));
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
            return ResultVOUtils.error(-1, jobLog.getMsg());
        }

        return ResultVOUtils.success();
    }

}
