package com.glxp.udidl.admin.service.dataSync;

import cn.hutool.core.util.StrUtil;
import com.glxp.udidl.admin.constant.Constant;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udid.*;
import com.glxp.udidl.admin.res.udid.DataSetResult;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.UdiCompanyService;
import com.glxp.udidl.admin.thread.UdiTransferUtils;
import com.glxp.udidl.admin.util.BeanUtils;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.IntUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * 同步数据存储
 */
@Service
public class DeviceSaveService {
    @Resource
    private DeviceService deviceService;
    @Resource
    private UdiTransferUtils udiTransferUtils;

    /**
     * 存储设备信息
     *
     * @param dataSets
     * @return 返回存储条数
     */
    public int DeviceSave(List<DataSetResult.DataSet> dataSets, Date requestDate, Integer dlType) {

        if (dlType == null) {       //默认新增
            dlType = Constant.DL_TYPE_ADD;
        }

        int result = 0;
        for (DataSetResult.DataSet ds : dataSets) {
            List<DataSetResult.ClinicalInfo> ClinicalInfoAll = ds.getClinicalInfo();
            List<DataSetResult.StorageInfo> storageInfoAll = ds.getStorageInfo();
            List<DataSetResult.PackingInfo> packingInfoAll = ds.getPackingInfo();
            if (ds.getDeviceInfo() != null)
                for (DataSetResult.DeviceInfo item : ds.getDeviceInfo()) {

                    //1:判断数据是否存在
                    String uuid = null;
                    Device originDevice = deviceService.selectDiIsSame(item.getZxxsdycpbs(), item.getVersionNumber() + "");
                    if (dlType == Constant.DL_TYPE_ADD) {
                        if (originDevice != null){
                            updateNewest(originDevice);
                            continue;
                        }

                    }


                    //2,存储 deviceInfo
                    Device device = responseToDevice(item);
                    Boolean isHistory = false;
                    if (item.getDeviceHistoryKey() != null && !item.getDeviceHistoryKey().isEmpty()) {
                        isHistory = true;
                    }

                    if (requestDate != null)
                        device.setRequestDate(requestDate);

                    //2.1 判断更新或者新增
                    if (dlType == Constant.DL_TYPE_ADD || originDevice == null) {
                        uuid = getUUId();
                        device.setUuid(uuid);
                        deviceService.insertDevice(device);
                    } else {
                        //2.2 判断字段是否与原来的一致，若不一致则更新
                        if (!device.equals(originDevice)) {
                            uuid = originDevice.getUuid();
                            device.setUuid(uuid);
                            deviceService.updateDevice(device);
                        } else { //，若一致则跳过
//                        uuid = getUUId();
//                        device.setUuid(uuid);
//                        deviceService.insertDevice(device);
                            updateNewest(originDevice);
                            continue;
                        }
                    }
                    result++;
                    //3,存储ContactList
                    List<DataSetResult.ContactInfo> contactInfos = item.getContactList();
                    if (contactInfos != null && contactInfos.size() > 0) {
                        List<Contactlist> contactlistList = new ArrayList<>();
                        for (DataSetResult.ContactInfo contactInfo : contactInfos) {
                            Contactlist contactlist = responseToContactlist(contactInfo);
                            contactlist.setDevicerecordkey(device.getDevicerecordkey());
                            contactlist.setUuid(uuid);
                            contactlistList.add(contactlist);
                        }
                        deviceService.deleteClinicalByUuid(uuid);
                        deviceService.insertContactlist(contactlistList);
                    }
                    //4,存储 Deviceclinical
                    if (ClinicalInfoAll != null && ClinicalInfoAll.size() > 0) {
                        List<DataSetResult.ClinicalInfo> deviceclinicals;
                        if (isHistory == true)
                            deviceclinicals = ClinicalInfoAll.stream().filter(p -> p.getDeviceHistoryKey().equals(item.getDeviceHistoryKey())).
                                    collect(Collectors.toList());
                        else
                            deviceclinicals = ClinicalInfoAll.stream().filter(p -> p.getDeviceRecordKey().equals(item.getDeviceRecordKey())).
                                    collect(Collectors.toList());
                        List<Deviceclinical> deviceclinicalList = new ArrayList<>();
                        if (deviceclinicals != null && deviceclinicals.size() > 0) {
                            for (DataSetResult.ClinicalInfo deviceClinical : deviceclinicals) {
                                Deviceclinical deviceclinical = responseToDeviceclinical(deviceClinical);
                                if (isHistory == true)
                                    deviceclinical.setDevicerecordkey(item.getDeviceHistoryKey());
                                else
                                    deviceclinical.setDevicerecordkey(device.getDevicerecordkey());
                                deviceclinical.setUuid(uuid);
                                deviceclinicalList.add(deviceclinical);
                            }
                            deviceService.deleteClinicalByUuid(uuid);
                            deviceService.insertDeviceClinical(deviceclinicalList);
                        }
                    }

                    //5 存储 Devicestorage
                    if (storageInfoAll != null && storageInfoAll.size() > 0) {
                        List<DataSetResult.StorageInfo> storageInfos;
                        if (isHistory == true)
                            storageInfos = storageInfoAll.stream().filter(p -> p.getDeviceHistoryKey().equals(item.getDeviceHistoryKey())).
                                    collect(Collectors.toList());
                        else
                            storageInfos = storageInfoAll.stream().filter(p -> p.getDeviceRecordKey().equals(item.getDeviceRecordKey())).
                                    collect(Collectors.toList());
                        if (storageInfos != null && storageInfos.size() > 0) {
                            List<Devicestorage> devicestorageList = new ArrayList<>();
                            for (DataSetResult.StorageInfo storageInfo : storageInfos) {
                                Devicestorage devicestorage = responseToDevicestorage(storageInfo);
                                if (isHistory == true)
                                    devicestorage.setDevicerecordkey(item.getDeviceHistoryKey());
                                else
                                    devicestorage.setDevicerecordkey(device.getDevicerecordkey());
                                devicestorage.setUuid(uuid);
                                devicestorageList.add(devicestorage);
                            }
                            deviceService.deleteStorageByUuid(uuid);
                            deviceService.insertDevicestorage(devicestorageList);
                        }

                    }

                    //6 存储Devicepackage
                    if (packingInfoAll != null && packingInfoAll.size() > 0) {
                        List<DataSetResult.PackingInfo> packingInfos;
                        if (isHistory == true)
                            packingInfos = packingInfoAll.stream().filter(p -> p.getDeviceHistoryKey().equals(item.getDeviceHistoryKey())).
                                    collect(Collectors.toList());
                        else
                            packingInfos = packingInfoAll.stream().filter(p -> p.getDeviceRecordKey().equals(item.getDeviceRecordKey())).
                                    collect(Collectors.toList());
                        if (packingInfos != null && packingInfos.size() > 0) {
                            List<Devicepackage> devicepackageList = new ArrayList<>();
                            for (DataSetResult.PackingInfo packingInfo : packingInfos) {
                                Devicepackage devicepackage = responseToDevicepackage(packingInfo);
                                if (isHistory == true)
                                    devicepackage.setDevicerecordkey(item.getDeviceHistoryKey());
                                else
                                    devicepackage.setDevicerecordkey(device.getDevicerecordkey());
                                devicepackage.setUuid(uuid);
                                devicepackageList.add(devicepackage);
                            }
                            deviceService.deletePakcageByUuid(uuid);
                            deviceService.insertDevicepackage(devicepackageList);
                        }

                    }

                    //7存储productInfo信息
                    productInfoSave(uuid);
                }
        }
        return result;
    }
    @Resource
    ProductInfoService productInfoService;

    public void updateNewest(Device device){
        if(!productInfoService.selectNewst(device.getDevicerecordkey(),true, IntUtil.value(device.getVersionnumber()))){
            productInfoService.updateNoNewst(device.getDevicerecordkey(),false,null);
            productInfoService.updateNoNewst(device.getDevicerecordkey(),true, IntUtil.value(device.getVersionnumber()));
        }

    }

    /**
     * device数据格式转换
     *
     * @param dataSetBean
     * @return
     */
    private Device responseToDevice(DataSetResult.DeviceInfo dataSetBean) {
        Device device = new Device();
        BeanUtils.copyProperties(dataSetBean, device);
        device.setDevicehistoryrecordkey(dataSetBean.getDeviceHistoryKey());
        device.setDevicerecordkey(dataSetBean.getDeviceRecordKey());
        device.setBtcpbs(dataSetBean.getBtcpbs());
        device.setBtcpbsyzxxsdycpbssfyz(dataSetBean.getBtcpbsyzxxsdycpbssfyz());
        device.setCgzmraqxgxx(dataSetBean.getCgzmraqxgxx());
        device.setCpbsbmtxmc(dataSetBean.getCpbsbmtxmc());
        device.setCpbsfbrq(dataSetBean.getCpbsfbrq());
        device.setCphhhbh(dataSetBean.getCphhhbh());
        device.setCplb(dataSetBean.getCplb());
        device.setCpmctymc(dataSetBean.getCpmctymc());
        device.setCpms(dataSetBean.getCpms());
        device.setFlbm(dataSetBean.getFlbm());
        device.setGgxh(dataSetBean.getGgxh());
        device.setMjfs(dataSetBean.getMjfs());
        device.setQtxxdwzlj(dataSetBean.getQtxxdwzlj());
        device.setQxlb(dataSetBean.getQxlb());
        device.setScbssfbhph(dataSetBean.getScbssfbhph());
        device.setScbssfbhscrq(dataSetBean.getScbssfbhscrq());
        device.setScbssfbhsxrq(dataSetBean.getScbssfbhsxrq());
        device.setScbssfbhxlh(dataSetBean.getScbssfbhxlh());
        device.setSfbjwycxsy(dataSetBean.getSfbjwycxsy());
        device.setSfwblztlcp(dataSetBean.getSfwblztlcp());
        device.setSfwwjbz(dataSetBean.getSfwwjbz());
        device.setSfybtzjbs(dataSetBean.getSfybtzjbs());
        device.setSpmc(dataSetBean.getSpmc());
        device.setSydycpbs(dataSetBean.getSydycpbs());
        device.setSyqsfxyjxmj(dataSetBean.getSyqsfxyjxmj());
        device.setTscchcztj(dataSetBean.getTscchcztj());
        device.setTsccsm(dataSetBean.getTsccsm());
        device.setTsrq(dataSetBean.getTsrq());
        device.setTyshxydm(dataSetBean.getTyshxydm());
        device.setVersionnumber(dataSetBean.getVersionNumber() + "");
        device.setVersionstatus(dataSetBean.getVersionStatus());
        device.setVersiontime(dataSetBean.getVersionTime());
        device.setYbbm(dataSetBean.getYbbm());
        device.setYlqxzcrbarmc(dataSetBean.getYlqxzcrbarmc());
        device.setYlqxzcrbarywmc(dataSetBean.getYlqxzcrbarywmc());
        device.setZczbhhzbapzbh(getString(dataSetBean.getZczbhhzbapzbh()));
        device.setZdcfsycs(dataSetBean.getZdcfsycs());
        device.setYflbm(dataSetBean.getYflbm());
        device.setZxxsdycpbs(dataSetBean.getZxxsdycpbs());
        device.setBszt(dataSetBean.getBszt());
        device.setSfyzcbayz(dataSetBean.getSfyzcbayz());
        device.setZcbacpbs(dataSetBean.getZcbacpbs());
        device.setZxxsdyzsydydsl(dataSetBean.getZxxsdyzsydydsl());
        device.setBssjzt(dataSetBean.getBssjzt());
        device.setLastModifyTime(DateUtil.formatDateTime(new Date()));
        device.setCplx(dataSetBean.getCplx());
        device.setHchzsb(dataSetBean.getHchzsb());
        return device;
    }

    private Contactlist responseToContactlist(DataSetResult.ContactInfo contactInfo) {
        Contactlist contactlist = new Contactlist();
        contactlist.setQylxrcz(contactInfo.getQylxrcz());
        contactlist.setQylxrdh(contactInfo.getQylxrdh());
        contactlist.setQylxryx(contactInfo.getQylxryx());
        return contactlist;
    }

    public Deviceclinical responseToDeviceclinical(DataSetResult.ClinicalInfo clinicalInfo) {
        Deviceclinical deviceclinical = new Deviceclinical();
        deviceclinical.setLcsycclx(clinicalInfo.getLcsycclx());
        deviceclinical.setCcz(clinicalInfo.getCcz());
        deviceclinical.setCcdw(clinicalInfo.getCcdw());
        return deviceclinical;
    }

    public Devicestorage responseToDevicestorage(DataSetResult.StorageInfo storageInfo) {
        Devicestorage devicestorage = new Devicestorage();
        devicestorage.setCchcztj(storageInfo.getCchcztj());
        devicestorage.setJldw(storageInfo.getJldw());
        devicestorage.setZgz(storageInfo.getZgz());
        devicestorage.setZdz(storageInfo.getZdz());
        return devicestorage;
    }

    public Devicepackage responseToDevicepackage(DataSetResult.PackingInfo packingInfo) {
        Devicepackage devicepackage = new Devicepackage();
        devicepackage.setBzcpbs(packingInfo.getBzcpbs());
        devicepackage.setBznhxyjbzcpbs(packingInfo.getBznhxyjbzcpbs());
        devicepackage.setCpbzjb(packingInfo.getCpbzjb());
        //devicepackage.setBznhxyjcpbssl(Integer.parseInt(packingInfo.getBznhxyjcpbssl()));
        devicepackage.setBznhxyjcpbssl(getInteger(packingInfo.getBznhxyjcpbssl()));
        return devicepackage;
    }

    private void productInfoSave(String uuid) {
        udiTransferUtils.transUdi(uuid);
    }

    private String getUUId() {
        UUID uuid = UUID.randomUUID();
        return uuid.toString().replace("-", "");
    }

    private String getString(String s) {
        if (s != null && s != "")
            return s.length() > 255 ? s.substring(0, 255) : s;
        else return s;
    }

    private Integer getInteger(String s) {
        try {
            return Integer.parseInt(s);
        } catch (Exception e) {
            return 0;
        }
    }
}
