package com.glxp.udidl.admin.service.dataSync;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.glxp.udidl.admin.constant.Constant;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.entity.udid.*;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.req.udid.DeviceRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.udid.DataSetResult;
import com.glxp.udidl.admin.res.udid.DataSetSingleHistoryResult;
import com.glxp.udidl.admin.res.udid.DataSetSingleResult;
import com.glxp.udidl.admin.res.udid.DataSetTotalResult;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.PushBuilder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class DeviceSyncService {

    @Resource
    private DeviceSaveService deviceSaveService;
    @Resource
    private DeviceDownloadService deviceDownloadService;
    @Resource
    JobLogService jobLogService;                //日志
    @Resource
    private ProductInfoService productInfoService;

    public void downloadAndSave(String day, String downloadType, Integer dlType) {
        //1.下载唯一标识数据
        if (dlType == null) {
            dlType = Constant.DL_TYPE_ADD;
        }
        DataSetTotalResult result;
        String rangeValue = "";
        if (day.isEmpty())
            rangeValue = DateUtil.getYesterday();
        else
            rangeValue = day;
        result = deviceDownloadService.getData(rangeValue, 1);
        if (result.getCode() != 1) {
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(downloadType);
            jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
            jobLog.setDownloadDate(DateUtil.parseDate(rangeValue));
            jobLog.setType(Constant.LOG_TYPE_ERROR);
            jobLog.setMsg(result.getMsg());
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
            return;
        }
        int insertCount = deviceSaveService.DeviceSave(result.getData(), DateUtil.parseDate(rangeValue), dlType);
        //2.下载历史记录
//        List<String> keyList = new ArrayList<String>();
//        for (DataSetResult.DataSet ds : result.getData()) {
//            List<DataSetResult.DeviceInfo> list = ds.getDeviceInfo();
//            if (list != null && list.size() > 0) {
//                for (DataSetResult.DeviceInfo deviceInfo : list)
//                    keyList.add(deviceInfo.getDeviceRecordKey());
//            }
//        }
//        if (keyList.size() > 0) {
//            DataSetTotalResult res = deviceDownloadService.getHistoryData2(keyList);
//            if (res.getCode() == 1) {
//                if (res.getData() != null && res.getData().size() > 0) {
//                    int count = result.getTotalRecordCount();
//                    count = count + res.getTotalRecordCount();
//                    result.setTotalRecordCount(count);
//                    List<DataSetResult.DataSet> data = result.getData();
//                    data.addAll(res.getData());
//                    result.setData(data);
//                    int hisInsertCount = deviceSaveService.DeviceSave(res.getData(), null, dlType);
//                    insertCount = insertCount + hisInsertCount;
//                }
//            } else {
//                JobLog jobLog = new JobLog();
//                jobLog.setDownloadType(downloadType);
//                jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
//                jobLog.setDownloadDate(DateUtil.parseDate(rangeValue));
//                jobLog.setType(Constant.LOG_TYPE_ERROR);
//                jobLog.setMsg("下载历史记录出错：" + res.getMsg());
//                jobLog.setCreateTime(new Date());
//                jobLogService.insert(jobLog);
//            }
//
//        }

        JobLog jobLog = new JobLog();
        jobLog.setDownloadType(downloadType);
        jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
        jobLog.setDownloadDate(DateUtil.parseDate(day));
        jobLog.setTotalCount(result.getTotalRecordCount());
        jobLog.setRealCount(result.realRecordCount());
        jobLog.setInsertCount(insertCount);
        if (result.realRecordCount() != result.getTotalRecordCount()) {
            jobLog.setType(Constant.LOG_TYPE_WARNING);
            jobLog.setMsg("接口总行数和实际行数不一致");
            jobLog.setCreateTime(new Date());
        } else {
            jobLog.setType(Constant.LOG_TYPE_INFO);
            jobLog.setMsg("执行成功！");
            jobLog.setCreateTime(new Date());
        }
        jobLogService.insert(jobLog);
    }

    @Async
    public void downloadUdi(String day, String downloadType) {
        try {
            downloadAndSave(day, downloadType, Constant.DL_TYPE_UPDATE);
        } catch (Exception e) {
            log.error("下载UDI国家库信息异常", e);
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(downloadType);
            jobLog.setDownloadDate(DateUtil.parseDate(day));
            jobLog.setType(Constant.LOG_TYPE_ERROR);
            jobLog.setMsg(e.getMessage());
            jobLog.setContent(JSON.toJSONString(e.getStackTrace()));
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        }
    }

    /**
     * 手动下载
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @Async
    public BaseResponse downloadUdi(Date startDate, Date endDate) {
        if (startDate.getTime() > endDate.getTime())
            return ResultVOUtils.error(-1, "起始日期不能大于结束日期!");
        Date fdate = startDate;
        while (fdate.getTime() <= endDate.getTime()) {
            String day = DateUtil.formatDate(fdate);
            downloadUdi(day, "manual");
            fdate = DateUtil.addDays(fdate, 1);
        }
        return ResultVOUtils.success("执行完成，详情请查看日志！");
    }

    public BaseResponse searchDlByDi(String deviceId) {
        List<ProductInfoEntity> productInfoEntityList = searchByDI(deviceId);
        if (CollUtil.isEmpty(productInfoEntityList)) {
            DataSetSingleResult result1 = deviceDownloadService.downloadByDi(deviceId);
            if (result1.getReturnCode() != 1)
                return ResultVOUtils.error(-1, result1.getReturnMsg());

            if (CollUtil.isNotEmpty(result1.getDataSet().getDeviceInfo())) {
                List<DataSetResult.DataSet> dataSets = new ArrayList<>();
                dataSets.add(result1.getDataSet());
                deviceSaveService.DeviceSave(dataSets, new Date(), Constant.DL_TYPE_UPDATE);
                productInfoEntityList = searchByDI(deviceId);
            } else {
                return ResultVOUtils.error(501, "医疗器械唯一标识数据库未找到此产品DI,请检查是否输入正确或联系厂家是否已上传");
            }

        }
        return ResultVOUtils.success(productInfoEntityList);
    }

    public BaseResponse searchDlLastByDi(String deviceId) {
        List<ProductInfoEntity> productInfoEntityList;
        DataSetSingleResult result1 = deviceDownloadService.downloadByDi(deviceId);
        if (result1.getReturnCode() != 1)
            return ResultVOUtils.error(-1, result1.getReturnMsg());

        if (CollUtil.isNotEmpty(result1.getDataSet().getDeviceInfo())) {
            List<DataSetResult.DataSet> dataSets = new ArrayList<>();
            dataSets.add(result1.getDataSet());
            deviceSaveService.DeviceSave(dataSets, new Date(), Constant.DL_TYPE_UPDATE);
            productInfoEntityList = searchByDI(deviceId);
        } else {
            return ResultVOUtils.error(501, "医疗器械唯一标识数据库未找到此产品DI,请检查是否输入正确或联系厂家是否已上传");
        }

        return ResultVOUtils.success(productInfoEntityList);
    }


    public List<ProductInfoEntity> searchByDI(String deviceId) {
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        productInfoFilterRequest.setNameCode(deviceId);
        productInfoFilterRequest.setIsNewest(1);
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        if (CollUtil.isNotEmpty(productInfoEntityList)) {
            ProductInfoFilterRequest request = new ProductInfoFilterRequest();
            request.setUuid(productInfoEntityList.get(0).getUuid());
            productInfoEntityList = productInfoService.findAll(request);
        }
        return productInfoEntityList;

    }

    public BaseResponse searchDlByUuid(String uuid) {
        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        productInfoFilterRequest.setUuid(uuid);
        List<ProductInfoEntity> productInfoEntityList = productInfoService.findAll(productInfoFilterRequest);
        return ResultVOUtils.success(productInfoEntityList);
    }


//    public void dlHistory(DataSetSingleResult result1) {
//
//        DataSetSingleHistoryResult result = new DataSetSingleHistoryResult();
//        result.setDataSet(result1.getDataSet());
//        if (result1.getDataSet().getDeviceInfo() != null && result1.getDataSet().getDeviceInfo().size() > 0) {
//            List<String> keyList = deviceDownloadService.getKeyList(result1.getDataSet());
//            if (keyList.size() > 0) {
//                DataSetTotalResult result2 = deviceDownloadService.getHistoryData(keyList);
//                result.setDataList(result2.getData());
//            }
//        }
//    }

    public BaseResponse downloadByDi(String deviceId) {
        DataSetSingleResult result1 = deviceDownloadService.downloadByDi(deviceId);
        if (result1.getReturnCode() != 1)
            return ResultVOUtils.error(-1, result1.getReturnMsg());
        DataSetSingleHistoryResult result = new DataSetSingleHistoryResult();
        result.setDataSet(result1.getDataSet());
//        if (result1.getDataSet().getDeviceInfo() != null && result1.getDataSet().getDeviceInfo().size() > 0) {
//            List<String> keyList = deviceDownloadService.getKeyList(result1.getDataSet());
//            if (keyList.size() > 0) {
//                DataSetTotalResult result2 = deviceDownloadService.getHistoryData(keyList);
//                result.setDataList(result2.getData());
//            }
//        }
        List<DataSetResult.DataSet> dataSets = new ArrayList<>();
        dataSets.add(result1.getDataSet());
        deviceSaveService.DeviceSave(dataSets, new Date(), Constant.DL_TYPE_UPDATE);
        return ResultVOUtils.success(result);
    }

    public BaseResponse downloadSingle(String deviceId, Integer dlType) {
        //插入日志
        JobLog jobLog = new JobLog();
        jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
        jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
        jobLog.setDownloadDate(new Date());
        jobLog.setType(Constant.LOG_TYPE_INFO);
        jobLog.setMsg("下载中");
        jobLog.setCreateTime(new Date());
        jobLog.setStatus(0);
        jobLogService.insertLog(jobLog);

        DataSetSingleResult res = null;
        try {
            res = deviceDownloadService.downloadByDi(deviceId);
            if (res.getReturnCode() != 1) {
                log.error("下载产品信息失败", res.getReturnMsg());
                jobLog.setMsg("下载失败：" + res.getReturnMsg());
                jobLog.setType(Constant.LOG_TYPE_ERROR);
                return ResultVOUtils.error(-1, res.getReturnMsg());
            }
            List<DataSetResult.DataSet> dataSets = new ArrayList<>();
            dataSets.add(res.getDataSet());
            int insertCount = deviceSaveService.DeviceSave(dataSets, new Date(), dlType);
            jobLog.setMsg("下载成功，总条数：" + dataSets.size() + " ；插入数据：" + insertCount + " 条");
            jobLog.setType(Constant.LOG_TYPE_SUCCESS);
            jobLogService.update(jobLog);
        } catch (Exception e) {
            log.error("下载产品信息失败", e);
            jobLog.setMsg("下载失败：" + res.getReturnMsg());
            jobLog.setType(Constant.LOG_TYPE_ERROR);
            jobLogService.update(jobLog);

            return ResultVOUtils.error(500, res.getReturnMsg());
        }
        return ResultVOUtils.success(Convert(res.getDataSet()));
    }

    /**
     * 下载单个历史记录，并转换格式
     *
     * @param key
     * @return
     */
    public BaseResponse downloadHistory(String key) {
        List<String> keyList = new ArrayList<>();
        keyList.add(key);
        DataSetTotalResult res = deviceDownloadService.getHistoryData1(keyList);
        if (res.getCode() != 1)
            return ResultVOUtils.error(-1, res.getMsg());
        List<DeviceEntity> result = new ArrayList<>();
        for (DataSetResult.DataSet ds : res.getData())
            result.addAll(Convert(ds));
        return ResultVOUtils.success(result);
    }

    /**
     * 格式转换
     *
     * @param dataSet
     * @return
     */
    private List<DeviceEntity> Convert(DataSetResult.DataSet dataSet) {
        List<DeviceEntity> result = new ArrayList<>();
        List<DataSetResult.DeviceInfo> deviceInfoList = dataSet.getDeviceInfo();
        List<DataSetResult.ClinicalInfo> clinicalInfos = dataSet.getClinicalInfo();
        List<DataSetResult.StorageInfo> storageInfos = dataSet.getStorageInfo();
        List<DataSetResult.PackingInfo> packingInfos = dataSet.getPackingInfo();

        if (deviceInfoList != null && deviceInfoList.size() > 0) {
            for (DataSetResult.DeviceInfo ds : deviceInfoList) {
                DeviceEntity deviceEntity = new DeviceEntity();
                BeanUtils.copyProperties(ds, deviceEntity);
                deviceEntity.setDevicerecordkey(ds.getDeviceRecordKey());
                deviceEntity.setVersionnumber(ds.getVersionNumber() + "");
                deviceEntity.setVersionstatus(ds.getVersionStatus());
                deviceEntity.setVersiontime(ds.getVersionTime());
                deviceEntity.setContactlistList(convertList2List(ds.getContactList(), Contactlist.class));
                if (clinicalInfos != null && clinicalInfos.size() > 0) {
                    List<DataSetResult.ClinicalInfo> clinicalInfos1 = clinicalInfos.stream().filter(p -> p.getDeviceRecordKey().equals(ds.getDeviceRecordKey())).
                            collect(Collectors.toList());
                    if (clinicalInfos1 != null && clinicalInfos1.size() > 0) {
                        deviceEntity.setDeviceclinicals(convertList2List(clinicalInfos1, Deviceclinical.class));
                        deviceEntity.getDeviceclinicals().forEach(p -> p.setDevicerecordkey(ds.getDeviceRecordKey()));
                    }

                }
                if (storageInfos != null && storageInfos.size() > 0) {
                    List<DataSetResult.StorageInfo> storageInfos1 = storageInfos.stream().filter(p -> p.getDeviceRecordKey().equals(ds.getDeviceRecordKey())).
                            collect(Collectors.toList());
                    if (storageInfos1 != null && storageInfos1.size() > 0) {
                        deviceEntity.setDevicestorages(convertList2List(storageInfos1, Devicestorage.class));
                        deviceEntity.getDevicestorages().forEach(p -> p.setDevicerecordkey(ds.getDeviceRecordKey()));
                    }

                }
                if (packingInfos != null && packingInfos.size() > 0) {
                    List<DataSetResult.PackingInfo> packingInfos1 = packingInfos.stream().filter(p -> p.getDeviceRecordKey().equals(ds.getDeviceRecordKey())).
                            collect(Collectors.toList());
                    if (packingInfos1 != null && packingInfos1.size() > 0) {
                        deviceEntity.setDevicepackages(convertList2List(packingInfos1, Devicepackage.class));
                        deviceEntity.getDevicepackages().forEach(p -> p.setDevicerecordkey(ds.getDeviceRecordKey()));
                    }

                }
                result.add(deviceEntity);
            }

        }
        return result;
    }

    private <E, T> List<T> convertList2List(List<E> input, Class<T> clzz) {
        List<T> output = new ArrayList<>();
        if (!CollectionUtils.isEmpty(input)) {
            for (E source : input) {
                T target = BeanUtils.instantiate(clzz);
                BeanUtils.copyProperties(source, target);
                output.add(target);
            }
        }
        return output;
    }

    /**
     * 下载从今天开始前几天的数据并保存  自动下载
     *
     * @param days
     */
    public void downloadUdi(int days) {
        for (int i = 1; i <= days; i++) {
            String day = DateUtil.getLastDay(-i);
            DeviceRequest deviceRequest = new DeviceRequest();
            deviceRequest.setRequestType("1"); //按天请求
            deviceRequest.setRangeValue(day);
            deviceRequest.setDownloadType("auto"); //下载方式
            deviceRequest.setDataType("3"); //全量下载
            downloadUdiData(deviceRequest);
        }
    }

    /**
     * 下载UDI国家库数据
     *
     * @param deviceRequest
     * @return
     */
    @Async
    public void downloadUdiData(DeviceRequest deviceRequest) {
        if (StrUtil.isBlank(deviceRequest.getRangeValue())) {
            switch (deviceRequest.getRequestType()) {
                case "1":
                    //按天下载
                    deviceRequest.setRangeValue(DateUtil.getDate());
                    break;
                case "2":
                    deviceRequest.setRangeValue(DateUtil.formatDate(new Date(), "yyyy-MM"));
                    break;
                case "3":
                    //当下载类型为时间段，但值为空时，默认下载当天的
                    deviceRequest.setRangeValue(DateUtil.getDate());
                    break;
                default:
                    deviceRequest.setRangeValue(DateUtil.getDate());
                    break;
            }
        }

        List<String> dates = new ArrayList<>();
        if (deviceRequest.getRequestType().equals("3")) {
            deviceRequest.setRequestType("1");
            //按时间段下载
            if (deviceRequest.getRangeValue().contains("--")) {
                String[] dateArr = deviceRequest.getRangeValue().split("--");
                int compareResult = DateUtil.compareDate(dateArr[0], dateArr[1], "yyyy-MM-dd");
                if (compareResult == -1) {
                    long between = cn.hutool.core.date.DateUtil.between(cn.hutool.core.date.DateUtil.parseDate(dateArr[0]), cn.hutool.core.date.DateUtil.parseDate(dateArr[1]), DateUnit.DAY);
                    between += 1; //包括自身
                    for (int i = 0; i < between; i++) {
                        dates.add(cn.hutool.core.date.DateUtil.offsetDay(DateUtil.parseDate(dateArr[0]), i).toString("yyyy-MM-dd"));
                    }
                } else if (compareResult == 0) {
                    dates.add(dateArr[0]);
                } else {
                    dates.add(DateUtil.getDate());
                }
            } else {
                dates.add(deviceRequest.getRangeValue());
            }
        } else {
            dates.add(deviceRequest.getRangeValue());
        }

        int dlType = Constant.DL_TYPE_ADD;

        if (StrUtil.isEmpty(deviceRequest.getDataType()) || deviceRequest.getDataType().equals("3")) {
            deviceRequest.setDataType(null);
        } else if (deviceRequest.getDataType().equals("1")) {
            dlType = Constant.DL_TYPE_ADD;
        } else if (deviceRequest.getDataType().equals("2")) {
            dlType = Constant.DL_TYPE_UPDATE;
        }
        if (deviceRequest.getDlType() != null) {
            dlType = deviceRequest.getDlType();
        }

        log.info("开始下载国家UDI库数据");
        log.info("本次任务开始下载时间：" + DateUtil.getDateTime());
        for (String date : dates) {
            log.info("下载：" + date + " 的数据");
            deviceRequest.setRangeValue(date);

            //插入日志
            JobLog jobLog = new JobLog();
            jobLog.setDownloadType(deviceRequest.getDownloadType());
            if (dlType == Constant.DL_TYPE_UPDATE) {
                jobLog.setTaskType(Constant.JOB_TASK_TYPE_UPDATE);
            } else
                jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
            jobLog.setDownloadDate(DateUtil.parseDate(date));
            jobLog.setType(Constant.LOG_TYPE_INFO);
            jobLog.setMsg("下载中");
            jobLog.setCreateTime(new Date());
            jobLog.setStatus(0);
            jobLogService.insertLog(jobLog);
            log.info(jobLog.getId() + "----");
            //开始下载数据
            boolean b = deviceDownloadService.getDataAndSave(deviceRequest, jobLog, dlType);

            if (b) {
                jobLog.setStatus(1);
                jobLog.setType(Constant.LOG_TYPE_SUCCESS);
                jobLog.setUpdateTime(new Date());
                jobLogService.update(jobLog);
            } else {
                jobLog.setStatus(1);
                jobLog.setType(Constant.LOG_TYPE_ERROR);
                jobLog.setUpdateTime(new Date());
                jobLogService.update(jobLog);
                continue;
            }
        }

        log.info("本次任务下载结束时间：" + DateUtil.getDateTime());
    }
}
