package com.glxp.udidl.admin.service.dataSync;


import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.util.ParameterizedTypeImpl;
import com.glxp.udidl.admin.config.GlobalConfig;
import com.glxp.udidl.admin.dto.udplat.UdplatDistributorModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsModel;
import com.glxp.udidl.admin.dto.udplat.UdplatHospitalModel;
import com.glxp.udidl.admin.dto.udplat.UdplatManufactureModel;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.udplat.UdplatResponse;
import com.glxp.udidl.admin.util.Md5Utils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import com.google.gson.reflect.TypeToken;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.lang.reflect.Type;
import java.net.URLEncoder;
import java.util.*;

@Service
@Slf4j
public class UdplatDownloadService {
    /* private String host="http://pre-mcs.udplat.org/mcs-api";//"http://120.32.125.20";//"http://pre.udplat.org/deal";"http://pre.udplat.org";//
     private String appId="20004072";
     private String secretKey="hoGxLSEsSyysnS9";
     private String userName="福建片仔癀诊断技术有限公司";//"漳州片仔癀诊断";*/
    @Autowired
    private GlobalConfig globalConfig;

    private long getTime() {
        return System.currentTimeMillis();
    }

    private String getUserName() {
        String userName = "";
        try {
            userName = URLEncoder.encode(globalConfig.getUdplat_userName(), "utf-8");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return userName;
    }

    /**
     * 获取签名
     *
     * @param mapParam
     * @return
     */
    private String getSign(Map<String, String> mapParam) {
        TreeMap<String, String> map = new TreeMap<>();

        map.put("appId", globalConfig.getUdplat_appId());
        map.put("userName", getUserName());
        map.put("secretKey", globalConfig.getUdplat_secretKey());
        map.putAll(mapParam);
        StringBuffer sb = new StringBuffer();
        for (Map.Entry<String, String> entry : map.entrySet()) {
            sb.append(entry.getKey() + "=" + entry.getValue());
            sb.append("&");
        }
        String str = sb.toString();
        String str0 = str.substring(0, str.length() - 1);
        log.info("userName：" + globalConfig.getUdplat_userName());
        log.info("签名前：" + str0);
        String result = Md5Utils.stringToMD5(str0);
        log.info("签名：" + result);
        return result;
    }

    private Map<String, String> getHeader(Map<String, String> mapParam) {
        Map<String, String> map = new HashMap<>();
        String time = getTime() + "";
        mapParam.put("time", time);
        map.put("appId", globalConfig.getUdplat_appId());
        map.put("userName", getUserName());
        map.put("time", time);
        map.put("secretKey", globalConfig.getUdplat_secretKey());
        map.put("sign", getSign(mapParam));
        return map;
    }

    public String getDistributor() {
        Map<String, String> map = new HashMap<>();
        String url = "/hsapi/purchase/distributors";
        String result = post3(url, getHeader(map), null, null);
        return result;
    }

    public BaseResponse getAllDistributors() {
        List<UdplatDistributorModel> result = new ArrayList<>();
        int page = 1;
        int totalPage = 1;
        int pageSize = 1000;
        while (page <= totalPage) {
            UdplatResponse response = getDistributors(page, pageSize);
            if (response.getSuccess() == false)
                return ResultVOUtils.error(-1, response.getMessage());
            totalPage = response.getData().getTotal() / pageSize + 1;
            List<UdplatDistributorModel> list = response.getData().getData();
            if (list.size() > 0) {
                result.addAll(list);
            }
            page++;
        }
        return ResultVOUtils.success(result);
    }

    /**
     * 下载全部生产厂家信息
     *
     * @return
     */
    public BaseResponse getAllManufactures() {
        List<UdplatManufactureModel> result = new ArrayList<>();
        int page = 1;
        int totalPage = 1;
        int pageSize = 1000;
        while (page <= totalPage) {
            UdplatResponse response = getManufactures(page, pageSize);
            if (response.getSuccess() == false)
                return ResultVOUtils.error(-1, response.getMessage());
            totalPage = response.getData().getTotal() / pageSize + 1;
            List<UdplatManufactureModel> list = response.getData().getData();
            if (list.size() > 0) {
                result.addAll(list);
            }
            page++;
        }
        return ResultVOUtils.success(result);
    }

    public BaseResponse getAllHospitals() {
        List<UdplatHospitalModel> result = new ArrayList<>();
        int page = 1;
        int totalPage = 1;
        int pageSize = 200;
        while (page <= totalPage) {
            UdplatResponse response = getHospitals(page, pageSize);
            if (response.getSuccess() == false)
                return ResultVOUtils.error(-1, response.getMessage());
            totalPage = response.getData().getTotal() / pageSize + 1;
            List<UdplatHospitalModel> list = response.getData().getData();
            if (list.size() > 0) {
                result.addAll(list);
            }
            page++;
        }
        return ResultVOUtils.success(result);
    }

    /**
     * 下载可选目录
     *
     * @param startTime
     * @param endTime
     * @return
     */
    public BaseResponse getAllUdplatGoods(String startTime, String endTime) {
        List<UdplatGoodsModel> result = new ArrayList<>();
        int page = 1;
        int totalPage = 1;
        int pageSize = 2000;
        while (page <= totalPage) {
            UdplatResponse response = getUdplatGoods(page, pageSize, startTime, endTime);
            if (response.getSuccess() == false)
                return ResultVOUtils.error(-1, response.getMessage());
            totalPage = response.getData().getTotal() / pageSize + 1;
            List<UdplatGoodsModel> list = response.getData().getData();
            if (list.size() > 0) {
                result.addAll(list);
            }
            page++;
        }
        return ResultVOUtils.success(result);
    }

    private UdplatResponse getManufactures(int page, int pageSize) {
        String url = "/hsapi/manufactures";
        Map<String, String> map = new HashMap<>();
        map.put("page", page + "");
        map.put("pageSize", pageSize + "");
        return post(UdplatManufactureModel.class, url, getHeader(map), null, map);
    }

    private UdplatResponse getDistributors(int page, int pageSize) {
        String url = "/hsapi/purchase/distributors";
        Map<String, String> map = new HashMap<>();
        map.put("page", page + "");
        map.put("pageSize", pageSize + "");
        return post(UdplatDistributorModel.class, url, getHeader(map), null, map);
    }

    private UdplatResponse getHospitals(int page, int pageSize) {
        String url = "/hsapi/distribute/hospitals";
        Map<String, String> map = new HashMap<>();
        map.put("page", page + "");
        map.put("pageSize", pageSize + "");
        return post(UdplatHospitalModel.class, url, getHeader(map), null, map);
    }

    public ResponseEntity<String> getPurchaseOrders(String orderCode) {
        String url = "/hsapi/purchase/orders";
        Map<String, String> map = new HashMap<>();
        if (orderCode != null && orderCode != "")
            map.put("orderCode", orderCode);
        return get(url, getHeader(map), null, map);
    }

    public ResponseEntity<String> getPurchaseOrderDetals(Long orderId) {
        String url = "/hsapi/purchase/orderDetals";
        Map<String, String> map = new HashMap<>();
        map.put("orderId", orderId + "");
        return get(url, getHeader(map), null, map);
    }

    public JSONObject getDistributeOrderDetails(Long orderId) {
        String url = "/hsapi/distribute/orderDetails";
        Map<String, String> map = new HashMap<>();
        map.put("orderId", orderId + "");
        return post2(url, getHeader(map), null, map);
    }

    public UdplatResponse getUdplatGoods(int page, int pageSize, String startTime, String endTime) {
        String url = "/hsapi/purchase/delivery/goods/query";
        Map<String, String> map = new HashMap<>();
        map.put("page", page + "");
        map.put("pageSize", pageSize + "");
        if (startTime != null && !startTime.isEmpty())
            map.put("startTime", startTime);
        if (endTime != null && !endTime.isEmpty())
            map.put("endTime", endTime);
        return get(UdplatGoodsModel.class, url, getHeader(map), null, map);
    }

    public ResponseEntity<String> getUdplatGoods(int page, int pageSize) {
        String url = "/hsapi/purchase/delivery/goods/query";
        Map<String, String> map = new HashMap<>();
        map.put("page", page + "");
        map.put("pageSize", pageSize + "");
        return get(url, getHeader(map), null, map);
    }

    public String getUdplatGoods2(int page, int pageSize) {
        String url = "/hsapi/distribute/delivery/goods/query";
        Map<String, String> map = new HashMap<>();
        map.put("page", page + "");
        map.put("pageSize", pageSize + "");
        return post3(url, getHeader(map), null, map);
    }

    private <T> ParameterizedTypeReference<UdplatResponse<T>> getReference2(Class<T> clazz) {

        Type wrapperType = TypeToken.getParameterized(UdplatResponse.class).getType();
        return ParameterizedTypeReference.forType(wrapperType);
    }

    private <T> UdplatResponse<T> post(Class<T> clazz, String url, Map<String, String> headerMap, Object jsonObject, Map<String, String> map) {
        HttpHeaders httpHeaders = new HttpHeaders();
        for (Map.Entry<String, String> stringStringEntry : headerMap.entrySet()) {
            httpHeaders.add(stringStringEntry.getKey(), stringStringEntry.getValue());
        }
        httpHeaders.add("Content-Type", "application/json");
        HttpEntity httpEntity = new HttpEntity(jsonObject, httpHeaders);
        RestTemplate restTemplate = new RestTemplate();
        String tmpUrl = url;
        if (map != null) {
            String params = "";
            for (Map.Entry<String, String> entry : map.entrySet()) {
                if (params == "")
                    params = "?" + entry.getKey() + "=" + entry.getValue();
                else
                    params = params + "&" + entry.getKey() + "=" + entry.getValue();
            }
            tmpUrl += params;
        }
        UdplatResponse<T> response = restTemplate.exchange(globalConfig.getUdplat_host() + tmpUrl, HttpMethod.POST, httpEntity, getReference2(clazz)).getBody();
        log.info(JSONObject.toJSONString(response));
        return response;
    }

    private <T> UdplatResponse<T> get(Class<T> clazz, String url, Map<String, String> headerMap, Object jsonObject, Map<String, String> map) {
        HttpHeaders httpHeaders = new HttpHeaders();
        for (Map.Entry<String, String> stringStringEntry : headerMap.entrySet()) {
            httpHeaders.add(stringStringEntry.getKey(), stringStringEntry.getValue());
        }
        httpHeaders.add("Content-Type", "application/json");
        HttpEntity httpEntity = new HttpEntity(jsonObject, httpHeaders);
        RestTemplate restTemplate = new RestTemplate();
        String tmpUrl = url;
        if (map != null) {
            String params = "";
            for (Map.Entry<String, String> entry : map.entrySet()) {
                if (params == "")
                    params = "?" + entry.getKey() + "=" + entry.getValue();
                else
                    params = params + "&" + entry.getKey() + "=" + entry.getValue();
            }
            tmpUrl += params;
        }
        UdplatResponse<T> response = restTemplate.exchange(globalConfig.getUdplat_host() + tmpUrl, HttpMethod.GET, httpEntity, getReference2(clazz)).getBody();
        log.info(JSONObject.toJSONString(response));
        return response;
    }

    private UdplatResponse post(String url, Map<String, String> headerMap, Object jsonObject, Map<String, String> map) {
        HttpHeaders httpHeaders = new HttpHeaders();
        for (Map.Entry<String, String> stringStringEntry : headerMap.entrySet()) {
            httpHeaders.add(stringStringEntry.getKey(), stringStringEntry.getValue());
        }
        httpHeaders.add("Content-Type", "application/json");
        HttpEntity httpEntity = new HttpEntity(jsonObject, httpHeaders);
        RestTemplate restTemplate = new RestTemplate();
        String tmpUrl = url;
        if (map != null) {
            String params = "";
            for (Map.Entry<String, String> entry : map.entrySet()) {
                if (params == "")
                    params = "?" + entry.getKey() + "=" + entry.getValue();
                else
                    params = params + "&" + entry.getKey() + "=" + entry.getValue();
            }
            tmpUrl += params;
        }
        return restTemplate.postForObject(globalConfig.getUdplat_host() + tmpUrl, httpEntity, UdplatResponse.class);
    }

    private String post3(String url, Map<String, String> headerMap, Object jsonObject, Map<String, String> map) {
        HttpHeaders httpHeaders = new HttpHeaders();
        for (Map.Entry<String, String> stringStringEntry : headerMap.entrySet()) {
            httpHeaders.add(stringStringEntry.getKey(), stringStringEntry.getValue());
        }
        httpHeaders.add("Content-Type", "application/json");
        HttpEntity httpEntity = new HttpEntity(jsonObject, httpHeaders);
        RestTemplate restTemplate = new RestTemplate();
        String tmpUrl = url;
        if (map != null) {
            String params = "";
            for (Map.Entry<String, String> entry : map.entrySet()) {
                if (params == "")
                    params = "?" + entry.getKey() + "=" + entry.getValue();
                else
                    params = params + "&" + entry.getKey() + "=" + entry.getValue();
            }
            tmpUrl += params;
        }
        String result = restTemplate.postForObject(globalConfig.getUdplat_host() + tmpUrl, httpEntity, String.class);
        return result;
    }

    private ResponseEntity<String> get(String url, Map<String, String> headerMap, Object jsonObject, Map<String, String> map) {
        HttpHeaders httpHeaders = new HttpHeaders();
        for (Map.Entry<String, String> stringStringEntry : headerMap.entrySet()) {
            httpHeaders.add(stringStringEntry.getKey(), stringStringEntry.getValue());
        }
        httpHeaders.add("Content-Type", "application/json");
        HttpEntity httpEntity = new HttpEntity(jsonObject, httpHeaders);
        RestTemplate restTemplate = new RestTemplate();
        String tmpUrl = url;
        if (map != null) {
            String params = "";
            for (Map.Entry<String, String> entry : map.entrySet()) {
                if (params == "")
                    params = "?" + entry.getKey() + "=" + entry.getValue();
                else
                    params = params + "&" + entry.getKey() + "=" + entry.getValue();
            }
            tmpUrl += params;
        }
        ResponseEntity<String> result = restTemplate.exchange(globalConfig.getUdplat_host() + tmpUrl, HttpMethod.GET, httpEntity, String.class);
        return result;
    }

    private JSONObject post2(String url, Map<String, String> headerMap, Object jsonObject, Map<String, String> map) {
        HttpHeaders httpHeaders = new HttpHeaders();
        for (Map.Entry<String, String> stringStringEntry : headerMap.entrySet()) {
            httpHeaders.add(stringStringEntry.getKey(), stringStringEntry.getValue());
        }
        httpHeaders.add("Content-Type", "application/json");
        HttpEntity httpEntity = new HttpEntity(jsonObject, httpHeaders);
        RestTemplate restTemplate = new RestTemplate();
        String tmpUrl = url;
        if (map != null) {
            String params = "";
            for (Map.Entry<String, String> entry : map.entrySet()) {
                if (params == "")
                    params = "?" + entry.getKey() + "=" + entry.getValue();
                else
                    params = params + "&" + entry.getKey() + "=" + entry.getValue();
            }
            tmpUrl += params;
        }
        JSONObject result = restTemplate.postForObject(globalConfig.getUdplat_host() + tmpUrl, httpEntity, JSONObject.class);
        return result;
    }

}
