package com.glxp.udidl.admin.service.dataSync;

import com.glxp.udidl.admin.dto.udplat.UdplatDistributorModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsModel;
import com.glxp.udidl.admin.dto.udplat.UdplatHospitalModel;
import com.glxp.udidl.admin.dto.udplat.UdplatManufactureModel;
import com.glxp.udidl.admin.entity.udplat.UdplatLog;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.udplat.*;
import com.glxp.udidl.admin.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class UdplatSyncService {
    @Autowired
    private UdplatDownloadService downloadService;
    @Autowired
    private UdplatLogService logService;
    @Autowired
    private UdplatManufactureService manufactureService;
    @Autowired
    private UdplatDistributorService distributorService;
    @Autowired
    private UdplatHospitalService hospitalService;
    @Autowired
    private UdplatGoodsService goodsService;

    @Async
    public void downloadAndSave(String downloadType, String tbName) {
        log.info("tbName:" + tbName);
        if (tbName == null || tbName == "" || tbName.equals("udplat_manufacture"))
            SyncManufacture(downloadType);
        if (tbName == null || tbName == "" || tbName.equals("udplat_distributor"))
            SyncDistributor(downloadType);
        if (tbName == null || tbName == "" || tbName.equals("udplat_hospital"))
            SyncHospitals(downloadType);
        if (tbName == null || tbName == "" || tbName.equals("udplat_goods"))
            SyncUdplatGoods(downloadType, null, null);
    }

    public void downloadByDay(String downloadType) {
        SyncManufacture(downloadType);
        SyncDistributor(downloadType);
        SyncHospitals(downloadType);
        String starTime = DateUtil.getYesterday();
        String endTime = DateUtil.getLastDay(1);
        SyncUdplatGoods(downloadType, starTime, endTime);
    }

    /**
     * 同步生产企业
     *
     * @param downloadType
     */
    private void SyncManufacture(String downloadType) {
        BaseResponse response = downloadService.getAllManufactures();
        if (response.getCode() != 20000) {
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType(downloadType);
            udplatLog.setTbName("udplat_manufacture");
            udplatLog.setType("error");
            udplatLog.setMsg(response.getMessage());
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return;
        }
        List<UdplatManufactureModel> list = (List<UdplatManufactureModel>) response.getData();
        manufactureService.save(list, downloadType);
    }

    /**
     * 同步配送企业
     *
     * @param downloadType
     */
    private void SyncDistributor(String downloadType) {
        BaseResponse response = downloadService.getAllDistributors();
        if (response.getCode() != 20000) {
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType(downloadType);
            udplatLog.setTbName("udplat_distributor");
            udplatLog.setType("error");
            udplatLog.setMsg(response.getMessage());
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return;
        }
        List<UdplatDistributorModel> list = (List<UdplatDistributorModel>) response.getData();
        distributorService.save(list, downloadType);
    }

    /**
     * 同步医疗机构清单
     *
     * @param downloadType
     */
    private void SyncHospitals(String downloadType) {
        BaseResponse response = downloadService.getAllHospitals();
        if (response.getCode() != 20000) {
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType(downloadType);
            udplatLog.setTbName("udplat_hospital");
            udplatLog.setType("error");
            udplatLog.setMsg(response.getMessage());
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return;
        }
        List<UdplatHospitalModel> list = (List<UdplatHospitalModel>) response.getData();
        hospitalService.save(list, downloadType);
    }

    /**
     * 同步可选目录
     *
     * @param downloadType
     * @param startTime
     * @param endTime
     */
    public void SyncUdplatGoods(String downloadType, String startTime, String endTime) {
        BaseResponse response = downloadService.getAllUdplatGoods(startTime, endTime);
        if (response.getCode() != 20000) {
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType(downloadType);
            udplatLog.setTbName("udplat_goods");
            udplatLog.setType("error");
            udplatLog.setMsg(response.getMessage());
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return;
        }
        List<UdplatGoodsModel> list = (List<UdplatGoodsModel>) response.getData();
        goodsService.save(list, downloadType);
    }
}
