package com.glxp.udidl.admin.service.dataUpdate;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.glxp.udidl.admin.constant.Constant;
import com.glxp.udidl.admin.entity.info.CompanyEntity;
import com.glxp.udidl.admin.entity.udid.JobLog;
import com.glxp.udidl.admin.entity.udid.TokenEntity;
import com.glxp.udidl.admin.req.UpdateUdiRequest;
import com.glxp.udidl.admin.req.udid.DownloadDiRequest;
import com.glxp.udidl.admin.req.udid.TokenRequest;
import com.glxp.udidl.admin.res.udid.DataSetResult;
import com.glxp.udidl.admin.res.udid.DataSetSingleResult;
import com.glxp.udidl.admin.service.info.CompanyService;
import com.glxp.udidl.admin.service.inout.DeviceService;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.util.HttpClient;
import com.glxp.udidl.admin.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class DeviceUpdateService {

    private String token;
    private String url = "sharing/get";

    @Resource
    private RedisUtil redisUtil;
    @Resource
    private CompanyService companyService;
    @Resource
    private JobLogService jobLogService;                //日志
    @Resource
    private DeviceParseService deviceParseService;

    public void updateUdiData(UpdateUdiRequest updateUdiRequest) {
        JobLog jobLog = new JobLog();
        jobLog.setCreateTime(new Date());
        jobLog.setStatus(0);
        jobLog.setType(Constant.LOG_TYPE_INFO);
        jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_MANUAL);
        jobLog.setTaskType(Constant.JOB_TASK_TYPE_UPDATE);
        jobLogService.insert(jobLog);

        if (updateUdiRequest.getType() == 0) {
            //按时间更新
            log.info("开始更新数据，更新时间段为： {} - {}", updateUdiRequest.getStartDate(), updateUdiRequest.getEndDate());
            //解析更新参数
            DateTime startDate = DateUtil.parse(updateUdiRequest.getStartDate(), DatePattern.SIMPLE_MONTH_PATTERN);
            DateTime endDate = DateUtil.parse(updateUdiRequest.getEndDate(), DatePattern.SIMPLE_MONTH_PATTERN);

            List<DateTime> dateRanges = DateUtil.rangeToList(startDate, endDate, DateField.MONTH);
            for (DateTime date : dateRanges) {

            }
        } else if (updateUdiRequest.getType() == 1) {
            jobLog.setTotalCount(1);
            jobLog.setRealCount(1);
            //按DI更新
            log.info("按照DI更新，更新的DI标识为：{}", updateUdiRequest.getNameCode());
            DataSetSingleResult dataSetSingleResult = downloadByDi(updateUdiRequest.getNameCode());
            if (dataSetSingleResult.getReturnCode() == -1) {
                log.error("更新DI信息异常" + dataSetSingleResult.getReturnMsg());
                jobLog.setType(Constant.LOG_TYPE_ERROR);
                jobLog.setMsg(dataSetSingleResult.getReturnMsg());
                jobLog.setInsertCount(0);
            } else {
                log.info("下载成功，开始更新数据");
                DataSetResult.DataSet dataSet = dataSetSingleResult.getDataSet();
                deviceParseService.DeviceSave(Collections.singletonList(dataSet), new Date());
                jobLog.setInsertCount(1);
            }
        }
        jobLog.setUpdateTime(new Date());
        jobLog.setStatus(1);
        jobLogService.update(jobLog);
    }


    /**
     * 获取当前token
     *
     * @return
     */
    public String getToken() {
        token = redisUtil.get("UDI_UPDATE_TOKEN");
        if (StrUtil.isBlank(token)) {
            //查询下载数据使用的appid
            CompanyEntity companyEntity = companyService.findByTaskType(Constant.TASK_TYPE_UPDATE);
            TokenRequest tokenRequest = new TokenRequest();
            tokenRequest.setAppId(companyEntity.getAppId());
            tokenRequest.setAppSecret(companyEntity.getAppSecret());
            tokenRequest.setTyshxydm(companyEntity.getTyshxydm());
            String response = HttpClient.post("token/get", tokenRequest);
            TokenEntity tokenEntity = JSONObject.parseObject(response, TokenEntity.class);
            token = tokenEntity.getAccessToken();
            redisUtil.setEx("UDI_UPDATE_TOKEN", token, 1 * 60 * 60 * 6);
        }
        return token;
    }

    /**
     * 获取产品标识详情数据
     *
     * @param deviceId
     * @return
     */
    public DataSetSingleResult downloadByDi(String deviceId) {
        DataSetSingleResult result = new DataSetSingleResult();
        DownloadDiRequest downloadDiRequest = new DownloadDiRequest();
        downloadDiRequest.setAccessToken(getToken());
        downloadDiRequest.setPrimaryDeviceId(deviceId);
        String response = "";
        try {
            response = HttpClient.post("sharing/single", downloadDiRequest);
        } catch (Exception e) {
            log.error("下载产品标识详情数据异常，响应结果：{}", response);
            result.setReturnCode(-1);
            result.setReturnMsg(e.getMessage());
            return result;//发生错误 退出
        }
        return JSONObject.parseObject(response, DataSetSingleResult.class);
    }

}
