package com.glxp.udidl.admin.service.info.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.glxp.udidl.admin.dao.info.CompanyMapper;
import com.glxp.udidl.admin.entity.info.CompanyEntity;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.info.CompanyService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import com.glxp.udidl.admin.util.UdiHttpClient;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
public class CompanyServiceImpl implements CompanyService {

    @Resource
    private CompanyMapper companyDao;
    @Resource
    private UdiHttpClient udiHttpClient;

    @Override
    public CompanyEntity findCompany() {
        return companyDao.findCompany();
    }

    @Override
    public CompanyEntity findByTaskType(Integer taskType) {
        return companyDao.findByTaskType(taskType);
    }

    @Override
    public CompanyEntity findByTyshxydm(String tyshxydm) {
        return companyDao.selectOne(new QueryWrapper<CompanyEntity>().eq("tyshxydm", tyshxydm));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean modifyCompany(CompanyEntity companyEntity) {
        if (findCompany() == null) {
            return companyDao.insertCompany(companyEntity);
        } else
            return companyDao.modifyCompany(companyEntity);
    }

    @Override
    public BaseResponse testConnection(String appId, String appSecret, String companyName, String creditNum, String dataSource) {
        CompanyEntity companyEntity = new CompanyEntity();
        companyEntity.setAppId(appId);
        companyEntity.setAppSecret(appSecret);
        companyEntity.setName(companyName);
        companyEntity.setTyshxydm(creditNum);
        String token = udiHttpClient.getToken(companyEntity);
        if (StrUtil.isBlank(token)) {
            return ResultVOUtils.error(500, "连接失败");
        }
        //查询数据库中是否存在此公司信息，存在则更新参数，不存在则添加新数据
        CompanyEntity data = companyDao.selectOne(new QueryWrapper<CompanyEntity>().eq("name", companyName).eq("tyshxydm", creditNum).eq("dataSource", dataSource));
        if (null == data) {
            companyEntity.setDataSource(dataSource);
            companyDao.insert(companyEntity);
        } else {
            data.setAppId(appId);
            data.setAppSecret(appSecret);
            //更新数据
            companyDao.update(data, new UpdateWrapper<CompanyEntity>().eq("tyshxydm", data.getTyshxydm()));
        }
        return ResultVOUtils.success("连接成功");
    }

    @Override
    public List<CompanyEntity> findCompanyListForToken() {
        QueryWrapper<CompanyEntity> queryParams = new QueryWrapper<>();
        queryParams.isNotNull("appId").isNotNull("appSecret").isNotNull("tyshxydm");
        return companyDao.selectList(queryParams);
    }
}
