package com.glxp.udidl.admin.service.inout.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.glxp.udidl.admin.dao.udi.ProductInfoMapper;
import com.glxp.udidl.admin.entity.udi.ProductInfoEntity;
import com.glxp.udidl.admin.req.ProductInfoFilterRequest;
import com.glxp.udidl.admin.service.inout.ProductInfoService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Transactional(rollbackFor = Exception.class)
@Service
public class ProductInfoServiceImpl implements ProductInfoService {

    @Resource
    ProductInfoMapper productInfoDao;

    @Override
    public List<ProductInfoEntity> filterCpmctymc(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
        PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        List<ProductInfoEntity> data = productInfoDao.filterCpmctymc(productInfoFilterRequest);
        return data;
    }

    @Override
    public List<ProductInfoEntity> filterProductInfo(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }

        List<ProductInfoEntity> data = productInfoDao.filterProductInfo(productInfoFilterRequest);
        return data;
    }

    @Override
    public List<ProductInfoEntity> filterUdiByTyshxydm(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        return productInfoDao.filterUdiByTyshxydm(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> filterUdiByCreditNo(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        return productInfoDao.filterUdiByCreditNo(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> filterUdiByNewest(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        return productInfoDao.filterUdiByNewest(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> findAllByUuid(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null && !productInfoFilterRequest.getPage().equals("")) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }
        List<String> uuids = productInfoDao.filterUuidByCreditNo(productInfoFilterRequest);
        if (uuids != null && uuids.size() > 0)
            return productInfoDao.selectAllByUuid(uuids);
        else
            return null;
    }

    @Override
    public List<ProductInfoEntity> filterUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        if (productInfoFilterRequest.getPage() != null) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            PageHelper.offsetPage(offset, productInfoFilterRequest.getLimit());
        }

        List<ProductInfoEntity> data = productInfoDao.filterUdi(productInfoFilterRequest);
        return data;
    }

    @Override
    public List<ProductInfoEntity> filterUdiNoPage(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest == null) {
            return Collections.emptyList();
        }
        if (productInfoFilterRequest.getPage() != null) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            if (offset < 0) {
                offset = 0;
            }
            productInfoFilterRequest.setPage(offset);
            productInfoFilterRequest.setLimit(productInfoFilterRequest.getLimit());
        } else {
            productInfoFilterRequest.setPage(1);
            productInfoFilterRequest.setLimit(10);
        }

        List<ProductInfoEntity> data = productInfoDao.filterUdiNoPage(productInfoFilterRequest);
        return data;
    }

    @Override
    public Long filterUdiNoPageCount(ProductInfoFilterRequest request) {


        ProductInfoFilterRequest productInfoFilterRequest = new ProductInfoFilterRequest();
        BeanUtils.copyProperties(request, productInfoFilterRequest);

        if (productInfoFilterRequest.getPage() != null) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit();
            productInfoFilterRequest.setPage(offset);
            productInfoFilterRequest.setLimit(productInfoFilterRequest.getLimit());

        }
        return productInfoDao.filterUdiNoPageCount(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> selectByUuid(String uuid) {
        List<ProductInfoEntity> data = productInfoDao.selectByUuid(uuid);
        return data;
    }

    @Override
    public List<ProductInfoEntity> syncDlUdi(ProductInfoFilterRequest productInfoFilterRequest) {
        if (productInfoFilterRequest.getPage() != null) {
            int offset = (productInfoFilterRequest.getPage() - 1) * productInfoFilterRequest.getLimit() + 1;
            productInfoFilterRequest.setPage(offset);
        }
        return productInfoDao.syncDlUdi(productInfoFilterRequest);
    }

    @Override
    public List<ProductInfoEntity> findAll(ProductInfoFilterRequest productInfoFilterRequest) {

        if (StrUtil.isEmpty(productInfoFilterRequest.getTyshxydm()) && StrUtil.isEmpty(productInfoFilterRequest.getNameCode())
                && StrUtil.isEmpty(productInfoFilterRequest.getYlqxzcrbarmc()) && StrUtil.isEmpty(productInfoFilterRequest.getDeviceRecordKey())
                && StrUtil.isEmpty(productInfoFilterRequest.getUuid())) {
            return Collections.emptyList();
        }
        List<ProductInfoEntity> data = productInfoDao.filterProductInfo(productInfoFilterRequest);
        return data;
    }

    @Override
    public List<ProductInfoEntity> findLastVersion(ProductInfoFilterRequest productInfoFilterRequest) {
        List<ProductInfoEntity> productInfoEntityList = findAll(productInfoFilterRequest);
        Map<String, List<ProductInfoEntity>> listMap = productInfoEntityList.stream().collect(Collectors.groupingBy(ProductInfoEntity::getUuid));
        List<ProductInfoEntity> result = null;
        for (List<ProductInfoEntity> value : listMap.values()) {
            if (result == null || value.get(0).getVersionNumber() > result.get(0).getVersionNumber()) {
                result = value;
            }
        }
        productInfoFilterRequest.setNameCode(null);
        productInfoFilterRequest.setUuid(result.get(0).getUuid());
        List<ProductInfoEntity> datas = findAll(productInfoFilterRequest);
        return datas;
    }

    public List<ProductInfoEntity> selectByUpdateTime(String startDate, String endDate) {
        return productInfoDao.selectByUpdateTime(startDate, endDate);
    }

    @Override
    public boolean insertProductInfo(ProductInfoEntity productInfoEntity) {
        return productInfoDao.insertProductInfo(productInfoEntity);
    }

    @Override
    public boolean updateProductByUuid(ProductInfoEntity productInfoEntity) {
        return productInfoDao.updateProductByUuid(productInfoEntity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateProductInfo(ProductInfoEntity productInfoEntity) {
        return productInfoDao.updateProductInfo(productInfoEntity);
    }


    @Override
    public boolean updateCplx(ProductInfoEntity productInfoEntity) {
        return productInfoDao.updateCplx(productInfoEntity);
    }

    @Override
    public boolean updateProductInfoData(ProductInfoEntity productInfoEntity) {
        return productInfoDao.updateProductInfoData(productInfoEntity);
    }

    @Override
    public boolean updateNoNewst(String deviceRecordKey, Boolean isNewst, Integer versionNum) {
        ProductInfoEntity productInfoEntity = new ProductInfoEntity();
        productInfoEntity.setIsNewest(isNewst);
        productInfoEntity.setUpdateTime(new Date());
        return productInfoDao.update(productInfoEntity,
                new QueryWrapper<ProductInfoEntity>().eq("deviceRecordKey",deviceRecordKey)
        .eq(versionNum!=null,"versionNumber",versionNum))>0;
    }

    @Override
    public boolean selectNewst(String deviceRecordKey, Boolean isNewest, Integer versionNum) {
        return productInfoDao.exists(new QueryWrapper<ProductInfoEntity>().eq("deviceRecordKey",deviceRecordKey)
        .eq("isNewest",isNewest).eq("versionNumber",versionNum));
    }

    @Override
    public boolean deleteById(String id) {
        return productInfoDao.deleteById(id);
    }

    @Override
    public boolean deleteAll(List<String> ids) {
        return productInfoDao.deleteAll(ids);
    }

}
