package com.glxp.udidl.admin.service.sys.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.sys.SysMenuMapper;
import com.glxp.udidl.admin.dto.sys.MenuTreeModel;
import com.glxp.udidl.admin.dto.sys.SysMenuModel;
import com.glxp.udidl.admin.entity.sys.SysMenu;
import com.glxp.udidl.admin.req.sys.SysMenuRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.sys.SysMenuService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SysMenuServiceImpl implements SysMenuService {

    @Resource
    SysMenuMapper sysMenuMapper;

    @Override
    public BaseResponse getMenuTree() {
        List<SysMenu> menus = sysMenuMapper.selectList(null);
        List<MenuTreeModel> result = getChild(menus, 0);
        return ResultVOUtils.success(result);
    }

    @Override
    public BaseResponse getMenuTreeActive() {
        List<SysMenu> menus = sysMenuMapper.selectActiveAll();
        List<MenuTreeModel> result = getChild(menus, 0);
        return ResultVOUtils.success(result);
    }

    @Transactional(rollbackFor = Exception.class)
    public BaseResponse insert(SysMenuModel model) {
        SysMenu sysMenu = new SysMenu();
        BeanUtils.copyProperties(model, sysMenu);
        sysMenu.setCreateTime(new Date());
        sysMenu.setUpdateTime(new Date());
        sysMenuMapper.insert(sysMenu);
        return ResultVOUtils.success();
    }

    @Transactional(rollbackFor = Exception.class)
    public BaseResponse update(SysMenuModel model) {
        SysMenu sysMenu = sysMenuMapper.selectById(model.getId());
        if (sysMenu == null)
            return ResultVOUtils.error(-1, "找不到记录");
        BeanUtils.copyProperties(model, sysMenu);
        sysMenu.setUpdateTime(new Date());
        sysMenuMapper.updateById(sysMenu);
        return ResultVOUtils.success();
    }

    public BaseResponse detail(Integer id) {
        SysMenu sysMenu = sysMenuMapper.selectById(id);
        if (sysMenu == null)
            return ResultVOUtils.error(-1, "找不到记录");
        return ResultVOUtils.success(sysMenu);
    }

    @Transactional(rollbackFor = Exception.class)
    public BaseResponse delete(Integer id) {
        if (sysMenuMapper.getChildCount(id) > 0)
            return ResultVOUtils.error(-1, "存在子级，请先删除子级！");
        sysMenuMapper.deleteById(id);
        return ResultVOUtils.success();
    }

    @Override
    public BaseResponse getList(SysMenuRequest sysMenuRequest) {
        QueryWrapper<SysMenu> queryWrapper = new QueryWrapper();
        queryWrapper.eq("type", sysMenuRequest.getType());
        if (null != sysMenuRequest.getParentId()) {
            queryWrapper.eq("parent_id", sysMenuRequest.getParentId());
        }
        PageHelper.startPage(sysMenuRequest.getPage(), sysMenuRequest.getLimit());
        List<SysMenu> sysMenus = sysMenuMapper.selectList(queryWrapper);
        PageInfo<SysMenu> pageInfo = new PageInfo<>(sysMenus);
        PageSimpleResponse<SysMenu> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(sysMenus);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    private List<MenuTreeModel> getChild(List<SysMenu> menus, Integer pId) {
        List<MenuTreeModel> result = new ArrayList<>();
        List<SysMenu> list = menus.stream().filter(p -> p.getParentId().equals(pId)).collect(Collectors.toList());
        if (list != null && list.size() > 0) {
            for (SysMenu item : list) {
                MenuTreeModel treeModel = new MenuTreeModel();
                BeanUtils.copyProperties(item, treeModel);
                List<MenuTreeModel> child = getChild(menus, treeModel.getId());
                if (child != null && child.size() > 0)
                    treeModel.setChildren(child);
                result.add(treeModel);
            }
        }
        return result;
    }
}
