package com.glxp.udidl.admin.service.sys.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.sys.SysRoleMapper;
import com.glxp.udidl.admin.dao.sys.SysRoleMenuMapper;
import com.glxp.udidl.admin.dto.sys.*;
import com.glxp.udidl.admin.entity.sys.SysRole;
import com.glxp.udidl.admin.entity.sys.SysRoleMenu;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.sys.SysMenuService;
import com.glxp.udidl.admin.service.sys.SysRoleService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
public class SysRoleServiceImpl implements SysRoleService {
    @Autowired
    SysRoleMapper mapper;
    @Autowired
    SysRoleMenuMapper sysRoleMenuMapper;
    @Autowired
    SysMenuService sysMenuService;

    @Override
    public BaseResponse list(SysRoleParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        List<SysRole> list = mapper.list(param);
        PageInfo<SysRole> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<SysRole> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse insert(SysRoleModel model) {
        SysRole sysRole = new SysRole();
        BeanUtils.copyProperties(model, sysRole);
        sysRole.setCreateTime(new Date());
        mapper.insert(sysRole);
        return ResultVOUtils.success();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse update(SysRoleModel model) {
        SysRole sysRole = mapper.selectByPrimaryKey(model.getId());
        if (sysRole == null)
            return ResultVOUtils.error(-1, "找不到记录");
        BeanUtils.copyProperties(model, sysRole);
        mapper.updateByPrimaryKey(sysRole);
        return ResultVOUtils.success();
    }

    @Override
    public BaseResponse detail(Integer id) {
        SysRole sysRole = mapper.selectByPrimaryKey(id);
        if (sysRole == null)
            return ResultVOUtils.error(-1, "找不到记录");
        return ResultVOUtils.success(sysRole);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse delete(Integer id) {
        mapper.deleteByPrimaryKey(id);
        return ResultVOUtils.success();
    }

    @Override
    public BaseResponse getRoleMenu(Integer roleId) {
        List<MenuTreeModel> menuTree = (List<MenuTreeModel>) sysMenuService.getMenuTreeActive().getData();
        List<Integer> list = sysRoleMenuMapper.selectByRoleId(roleId);
        return ResultVOUtils.success(RoleMenuTreeModel.builder().menuTrees(menuTree).roleMenuIds(list).build());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse saveRoleMenu(SysRoleMenuModel model) {
        sysRoleMenuMapper.deleteByRoleId(model.getRoleId());
        for (Integer menuId : model.getMenuIds()) {
            SysRoleMenu roleMenu = new SysRoleMenu();
            roleMenu.setRoleId(model.getRoleId());
            roleMenu.setMenuId(menuId);
            sysRoleMenuMapper.insert(roleMenu);
        }
        return ResultVOUtils.success();
    }
}
