package com.glxp.udidl.admin.service.sys.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.sys.SysUserMapper;
import com.glxp.udidl.admin.dto.sys.SysUserModel;
import com.glxp.udidl.admin.dto.sys.SysUserParam;
import com.glxp.udidl.admin.entity.sys.SysUser;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.sys.SysUserService;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Service
public class SysUserServiceImpl implements SysUserService {

    @Resource
    SysUserMapper sysUserMapper;

    @Override
    public BaseResponse list(SysUserParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        List<SysUser> list;
        if (null != param && StrUtil.isNotBlank(param.getStatus())) {
            list = sysUserMapper.selectList(new QueryWrapper<SysUser>().eq("status", param.getStatus()).orderByDesc("updateTime"));
        } else {
            list = sysUserMapper.selectList(new QueryWrapper<SysUser>().orderByDesc("updateTime"));
        }
        PageInfo<SysUser> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<SysUser> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse insert(SysUserModel model) {
        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(model, sysUser);
        Date date = new Date();
        sysUser.setCreateTime(date);
        sysUser.setUpdateTime(date);
        sysUserMapper.insert(sysUser);
        return ResultVOUtils.success();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse update(SysUserModel model) {
        SysUser sysUser = sysUserMapper.selectById(model.getId());
        if (sysUser == null)
            return ResultVOUtils.error(-1, "找不到记录");
        BeanUtils.copyProperties(model, sysUser);
        sysUser.setUpdateTime(new Date());
        sysUserMapper.updateById(sysUser);
        return ResultVOUtils.success();
    }

    @Override
    public BaseResponse detail(Integer id) {
        SysUser sysUser = sysUserMapper.selectById(id);
        if (sysUser == null)
            return ResultVOUtils.error(-1, "找不到记录");
        return ResultVOUtils.success(sysUser);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse delete(Integer id) {
        sysUserMapper.deleteByPrimaryKey(id);
        return ResultVOUtils.success();
    }

}
