package com.glxp.udidl.admin.service.udi.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.udid.JobLogMapper;
import com.glxp.udidl.admin.entity.udid.JobLog;
import com.glxp.udidl.admin.req.udid.JobLogFilterRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.ResultVOUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class JobLogServiceImpl implements JobLogService {

    @Resource
    private JobLogMapper jobLogMapper;

    public BaseResponse getList(JobLogFilterRequest param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        if (param.getEndDate() != null && !param.getEndDate().isEmpty())
            param.setEndDate(DateUtil.formatDate(DateUtil.addDays(DateUtil.parseDate(param.getEndDate()), 1)));
        List<JobLog> list = jobLogMapper.list(param);
        PageInfo<JobLog> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<JobLog> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Override
    public JobLog findById(String jobId) {
        return jobLogMapper.selectById(jobId);
    }


    public int insert(JobLog jobLog) {
        return jobLogMapper.insert(jobLog);
    }

    public int update(JobLog jobLog) {
        return jobLogMapper.updateById(jobLog);
    }

    @Override
    public int insertLog(JobLog jobLog) {
        int insert = jobLogMapper.insert(jobLog);
        return insert;
    }

    @Override
    public int deleteById(String jobId) {
        return jobLogMapper.deleteById(jobId);
    }

}
