package com.glxp.udidl.admin.service.udplat.impl;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.udplat.UdplatDistributorMapper;
import com.glxp.udidl.admin.dto.udplat.BaseParam;
import com.glxp.udidl.admin.dto.udplat.UdplatDistributorModel;
import com.glxp.udidl.admin.dto.udplat.UnitsListModel;
import com.glxp.udidl.admin.dto.udplat.UnitsParam;
import com.glxp.udidl.admin.entity.udplat.UdplatDistributor;
import com.glxp.udidl.admin.entity.udplat.UdplatLog;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.udplat.UdplatDistributorService;
import com.glxp.udidl.admin.service.udplat.UdplatLogService;
import com.glxp.udidl.admin.util.BeanUtils;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.FileUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

@Service
public class UdplatDistributorServiceImpl implements UdplatDistributorService {
    @Autowired
    private UdplatDistributorMapper mapper;
    @Autowired
    private UdplatLogService logService;

    @Override
    public BaseResponse getList(BaseParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        if (param.getEndDate() != null && !param.getEndDate().isEmpty())
            param.setEndDate(DateUtil.formatDate(DateUtil.addDays(DateUtil.parseDate(param.getEndDate()), 1)));
        List<UdplatDistributor> list = mapper.list(param);
        PageInfo<UdplatDistributor> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<UdplatDistributor> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse save(List<UdplatDistributorModel> models, String type) {
        if (models != null && models.size() > 0) {
            for (UdplatDistributorModel item : models) {
                UdplatDistributor entity = mapper.selectByDistributorId(item.getDistributorId());
                if (entity == null) {
                    entity = new UdplatDistributor();
                    BeanUtils.copyProperties(item, entity);
                    entity.setCreateTime(new Date());
                    mapper.insert(entity);
                } else {
                    BeanUtils.copyProperties(item, entity);
                    entity.setUpdateTime(new Date());
                    mapper.updateByPrimaryKey(entity);
                }
            }
        }
        UdplatLog udplatLog = new UdplatLog();
        udplatLog.setDownloadType(type);
        udplatLog.setTbName("udplat_distributor");
        udplatLog.setType("info");
        udplatLog.setMsg("执行成功！");
        udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
        udplatLog.setTotalCount(models.size());
        udplatLog.setCreateTime(new Date());
        logService.insert(udplatLog);
        return ResultVOUtils.success();
    }

    @Override
    public void exportToFile(HttpServletResponse response) {
        List<UdplatDistributor> list = mapper.selectAll();
        FileUtils.exportToFile(response, JSONObject.toJSONString(list), "配送企业");
    }

    public BaseResponse getUnitsList(UnitsParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        List<UnitsListModel> list = mapper.getUnitsList(param.getKey());
        PageInfo<UnitsListModel> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<UnitsListModel> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }
}
