package com.glxp.udidl.admin.service.udplat.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.udi.ProductInfoMapper;
import com.glxp.udidl.admin.dao.udplat.UdplatGoodsMatchMapper;
import com.glxp.udidl.admin.dto.device.ProductDetailModel;
import com.glxp.udidl.admin.dto.device.ProductGoodsMatchModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsMatchListModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsMatchModel;
import com.glxp.udidl.admin.dto.udplat.UdplatGoodsMatchParam;
import com.glxp.udidl.admin.entity.udid.UdiEntity;
import com.glxp.udidl.admin.entity.udplat.UdplatGoodsMatch;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.udplat.UdplatGoodsMatchService;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.FilterUdiUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Service
public class UdplatGoodsMatchServiceImpl implements UdplatGoodsMatchService {

    @Autowired
    private UdplatGoodsMatchMapper udplatGoodsMatchMapper;
    @Autowired
    private ProductInfoMapper productInfoDao;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse save(UdplatGoodsMatchModel model) {
        UdplatGoodsMatch udplatGoodsMatch = udplatGoodsMatchMapper.selectByDeviceRecordKey(model.getDeviceRecordKey());
        if (udplatGoodsMatch == null) {
            udplatGoodsMatch = new UdplatGoodsMatch();
            BeanUtils.copyProperties(model, udplatGoodsMatch);
            udplatGoodsMatch.setCreateTime(new Date());
            udplatGoodsMatchMapper.insert(udplatGoodsMatch);
        } else {
            udplatGoodsMatch.setDeliveryGoodsId(model.getDeliveryGoodsId());
            udplatGoodsMatch.setMedicalCode(model.getMedicalCode());
            udplatGoodsMatch.setUpdateTime(new Date());
            udplatGoodsMatchMapper.updateByPrimaryKey(udplatGoodsMatch);
        }
        return ResultVOUtils.success();
    }

    public BaseResponse getList(UdplatGoodsMatchParam param) {
        String key = param.getUdi();
        UdiEntity udiEntity = null;
        if (key != null && key != "") {
            udiEntity = FilterUdiUtils.getUdi(key);
            key = udiEntity.getUdi();
            param.setNameCode(key);
        }
        PageHelper.startPage(param.getPage(), param.getLimit());
        if (param.getEndDate() != null && !param.getEndDate().isEmpty())
            param.setEndDate(DateUtil.formatDate(DateUtil.addDays(DateUtil.parseDate(param.getEndDate()), 1)));
        List<UdplatGoodsMatchListModel> list = udplatGoodsMatchMapper.getList(param);
        PageInfo<UdplatGoodsMatchListModel> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<UdplatGoodsMatchListModel> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    public BaseResponse getProductList(String key) {
        List<ProductDetailModel> list = productInfoDao.selectByDeviceRecordKey(key);
        return ResultVOUtils.success(list);
    }

    public BaseResponse getProductInfoByUuid(String uuid) {
        List<ProductGoodsMatchModel> productInfoEntityList = udplatGoodsMatchMapper.getMatchList(uuid);//productInfoDao.selectByUuid(uuid);
        if (productInfoEntityList == null || productInfoEntityList.size() < 1)
            return ResultVOUtils.error(-1, "查无记录！");
        else
            return ResultVOUtils.success(productInfoEntityList.get(0));
    }
}
