package com.glxp.udidl.admin.service.udplat.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.udplat.UdplatGoodsMapper;
import com.glxp.udidl.admin.dto.udplat.*;
import com.glxp.udidl.admin.entity.udplat.UdplatGoods;
import com.glxp.udidl.admin.entity.udplat.UdplatLog;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.info.FileInfoService;
import com.glxp.udidl.admin.service.udplat.UdplatGoodsService;
import com.glxp.udidl.admin.service.udplat.UdplatLogService;
import com.glxp.udidl.admin.util.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class UdplatGoodsServiceImpl implements UdplatGoodsService {

    @Resource
    private UdplatGoodsMapper udplatGoodsMapper;
    @Resource
    private UdplatLogService logService;
    @Resource
    private FileInfoService fileInfoService;

    public BaseResponse getList(UdplatGoodsParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        if (param.getEndDate() != null && !param.getEndDate().isEmpty())
            param.setEndDate(DateUtil.formatDate(DateUtil.addDays(DateUtil.parseDate(param.getEndDate()), 1)));
        List<UdplatGoods> list = udplatGoodsMapper.list(param);
        PageInfo<UdplatGoods> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<UdplatGoods> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Override
    public BaseResponse importExcel(InputStream is, int type) {
        int count = 0;
        try {
            if (type == 1) {
                List<HospitalGoods> list1 = getData1(is);
                if (list1 == null || list1.size() < 1)
                    return ResultVOUtils.error(-1, "无数据！");
                count = SaveHospitalGoods(list1);
            } else if (type == 2) {
                List<DeliveryGoods> list2 = getData2(is);
                if (list2 == null || list2.size() < 1)
                    return ResultVOUtils.error(-1, "无数据！");
                count = SaveDeliveryGoods(list2);
            } else
                return ResultVOUtils.error(-1, "文件类型不匹配！");
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType("import");
            udplatLog.setTbName("udplat_goods");
            udplatLog.setType("info");
            udplatLog.setMsg("执行成功！");
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setTotalCount(count);
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return ResultVOUtils.success(count);
        } catch (Exception e) {
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType("import");
            udplatLog.setTbName("udplat_goods");
            udplatLog.setType("error");
            udplatLog.setMsg(e.getMessage());
            udplatLog.setContent(JSON.toJSONString(e.getStackTrace()));
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
            return ResultVOUtils.error(-1, "出错了:" + e.getMessage());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public BaseResponse save(List<UdplatGoodsModel> models, String downloadType) {
        if (models.size() > 0)
            try {
                for (UdplatGoodsModel item : models) {
                    String deliveryGoodsId = item.getDeliveryGoodsId();
                    UdplatGoods goods = udplatGoodsMapper.selectByGoodsId(deliveryGoodsId);
                    if (goods == null) {
                        goods = new UdplatGoods();
                        goods.setId(0L);
                    }
                    BeanUtils.copyProperties(item, goods);
                    if (goods.getId() > 0) {
                        goods.setUpdateTime(new Date());
                        udplatGoodsMapper.update(goods);
                    } else {
                        goods.setUuid(BeanUtils.getUUId());
                        goods.setCreateTime(new Date());
                        udplatGoodsMapper.insert(goods);
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                log.error(JSON.toJSONString(e.getStackTrace()));
                UdplatLog udplatLog = new UdplatLog();
                udplatLog.setDownloadType(downloadType);
                udplatLog.setTbName("udplat_goods");
                udplatLog.setType("error");
                udplatLog.setMsg(e.getMessage());
                udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
                udplatLog.setCreateTime(new Date());
                logService.insert(udplatLog);
                return ResultVOUtils.error(-1, e.getMessage());
            }
        UdplatLog udplatLog = new UdplatLog();
        udplatLog.setDownloadType(downloadType);
        udplatLog.setTbName("udplat_goods");
        udplatLog.setType("info");
        udplatLog.setMsg("执行成功！");
        udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
        udplatLog.setTotalCount(models.size());
        udplatLog.setCreateTime(new Date());
        logService.insert(udplatLog);
        return ResultVOUtils.success();
    }

    private List<HospitalGoods> getData1(InputStream is) throws Exception {
        ExcelUtil<HospitalGoods> util = new ExcelUtil(HospitalGoods.class);
        return util.importExcel(is);
    }

    private List<DeliveryGoods> getData2(InputStream is) throws Exception {
        ExcelUtil<DeliveryGoods> util = new ExcelUtil(DeliveryGoods.class);
        return util.importExcel(is);
    }

    public BaseResponse importHospitalGoods(List<HospitalGoods> list) {
        int count = SaveHospitalGoods(list);
        UdplatLog udplatLog = new UdplatLog();
        udplatLog.setDownloadType("import");
        udplatLog.setTbName("udplat_goods");
        udplatLog.setType("info");
        udplatLog.setMsg("执行成功！");
        udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
        udplatLog.setTotalCount(count);
        udplatLog.setCreateTime(new Date());
        logService.insert(udplatLog);
        return ResultVOUtils.success(count);
    }

    public BaseResponse importDeliveryGoods(List<DeliveryGoods> list) {
        int count = SaveDeliveryGoods(list);
        UdplatLog udplatLog = new UdplatLog();
        udplatLog.setDownloadType("import");
        udplatLog.setTbName("udplat_goods");
        udplatLog.setType("info");
        udplatLog.setMsg("执行成功！");
        udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
        udplatLog.setTotalCount(count);
        udplatLog.setCreateTime(new Date());
        logService.insert(udplatLog);
        return ResultVOUtils.success(count);
    }

    @Transactional(rollbackFor = Exception.class)
    private int SaveHospitalGoods(List<HospitalGoods> list) {
        int count = 0;
        for (HospitalGoods item : list) {
            String projectGoodsCode = item.getProjectGoodsCode();
            if (projectGoodsCode == null || projectGoodsCode.isEmpty())
                continue;
            UdplatGoods goods = udplatGoodsMapper.selectByProjectGoodsCode(projectGoodsCode);
            goods = HospitalGoodsToUdplatGoods(item, goods);
            if (goods.getId() > 0) {
                goods.setUpdateTime(new Date());
                udplatGoodsMapper.update(goods);
            } else {
                goods.setUuid(BeanUtils.getUUId());
                goods.setCreateTime(new Date());
                udplatGoodsMapper.insert(goods);
            }
            count++;
        }
        return count;
    }

    @Transactional(rollbackFor = Exception.class)
    private int SaveDeliveryGoods(List<DeliveryGoods> list) {
        int count = 0;
        for (DeliveryGoods item : list) {
            String projectGoodsCode = item.getProjectGoodsCode();
            if (projectGoodsCode == null || projectGoodsCode.isEmpty())
                continue;
            UdplatGoods goods = udplatGoodsMapper.selectByProjectGoodsCode(projectGoodsCode);
            goods = DeliveryGoodsToUdplatGoods(item, goods);
            if (goods.getId() > 0L) {
                goods.setUpdateTime(new Date());
                udplatGoodsMapper.update(goods);
            } else {
                goods.setUuid(BeanUtils.getUUId());
                goods.setCreateTime(new Date());
                udplatGoodsMapper.insert(goods);
            }
            count++;
        }
        return count;
    }

    private UdplatGoods HospitalGoodsToUdplatGoods(HospitalGoods hospitalGoods, UdplatGoods udplatGoods) {
        if (udplatGoods == null) {
            udplatGoods = new UdplatGoods();
            udplatGoods.setId(0L);
        }
        udplatGoods.setProjectGoodsCode(hospitalGoods.getProjectGoodsCode());
        udplatGoods.setManufactureName(hospitalGoods.getManufactureName());
        udplatGoods.setDistributorName(hospitalGoods.getDistributorName());
        udplatGoods.setProductName(hospitalGoods.getProductName());
        udplatGoods.setModel(hospitalGoods.getModel());
        udplatGoods.setSpec(hospitalGoods.getSpec());
        udplatGoods.setProdMaterial(hospitalGoods.getProdMaterial());
        udplatGoods.setRegNum(hospitalGoods.getRegNum());
        udplatGoods.setRegName(hospitalGoods.getRegName());
        udplatGoods.setRegValidTo(hospitalGoods.getRegValidTo());
        udplatGoods.setSalePrice(hospitalGoods.getSalePrice());
        udplatGoods.setPriceUnitText(hospitalGoods.getPriceUnitText());
        udplatGoods.setChangedContent(hospitalGoods.getChangedContent());
        udplatGoods.setChangedTime(hospitalGoods.getChangedTime());
        udplatGoods.setMedicalCode(hospitalGoods.getMedicalCode());
        return udplatGoods;
    }

    private UdplatGoods DeliveryGoodsToUdplatGoods(DeliveryGoods deliveryGoods, UdplatGoods udplatGoods) {
        if (udplatGoods == null) {
            udplatGoods = new UdplatGoods();
            udplatGoods.setId(0L);
        }
        udplatGoods.setProjectGoodsCode(deliveryGoods.getProjectGoodsCode());
        udplatGoods.setProductName(deliveryGoods.getProductName());
        udplatGoods.setModel(deliveryGoods.getModel());
        udplatGoods.setSpec(deliveryGoods.getSpec());
        udplatGoods.setProdMaterial(deliveryGoods.getProdMaterial());
        udplatGoods.setPackMaterial(deliveryGoods.getPackMaterial());
        udplatGoods.setRegNum(deliveryGoods.getRegNum());
        udplatGoods.setRegName(deliveryGoods.getRegName());
        udplatGoods.setRegValidTo(deliveryGoods.getRegValidTo());
        udplatGoods.setStockStatus(deliveryGoods.getStockStatus());
        udplatGoods.setChangedContent(deliveryGoods.getChangedContent());
        udplatGoods.setChangedTime(deliveryGoods.getChangedTime());
        udplatGoods.setMedicalCode(deliveryGoods.getMedicalCode());
        return udplatGoods;
    }

    @Override
    public void exportToFile(HttpServletResponse response) {
        List<UdplatGoods> list = udplatGoodsMapper.selectAll();
        FileUtils.exportToFile(response, JSONObject.toJSONString(list), "产品目录");
    }

    @Override
    public BaseResponse exportToFile(UdplatGoodsParam param) {
        if (param.getEndDate() != null && !param.getEndDate().isEmpty())
            param.setEndDate(DateUtil.formatDate(DateUtil.addDays(DateUtil.parseDate(param.getEndDate()), 1)));
        List<UdplatGoods> list = udplatGoodsMapper.list(param);
        return fileInfoService.saveToFile(JSONObject.toJSONString(list), "产品目录.json");
    }

    public BaseResponse selectById(Integer id) {
        UdplatGoods result = udplatGoodsMapper.selectById(id);
        if (result == null)
            return ResultVOUtils.error(-1, "找不到记录！");
        return ResultVOUtils.success(result);
    }

    public BaseResponse getProducts(ProductParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        List<ProductListModel> list = udplatGoodsMapper.getProducts(param);
        PageInfo<ProductListModel> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<ProductListModel> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }
}
