package com.glxp.udidl.admin.service.udplat.impl;


import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.glxp.udidl.admin.dao.udplat.UdplatHospitalMapper;
import com.glxp.udidl.admin.dto.udplat.BaseParam;
import com.glxp.udidl.admin.dto.udplat.UdplatHospitalModel;
import com.glxp.udidl.admin.entity.udplat.UdplatHospital;
import com.glxp.udidl.admin.entity.udplat.UdplatLog;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.res.PageSimpleResponse;
import com.glxp.udidl.admin.service.udplat.UdplatHospitalService;
import com.glxp.udidl.admin.service.udplat.UdplatLogService;
import com.glxp.udidl.admin.util.BeanUtils;
import com.glxp.udidl.admin.util.DateUtil;
import com.glxp.udidl.admin.util.FileUtils;
import com.glxp.udidl.admin.util.ResultVOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;

@Service
public class UdplatHospitalServiceImpl implements UdplatHospitalService {

    @Autowired
    private UdplatHospitalMapper mapper;
    @Autowired
    private UdplatLogService logService;

    @Override
    public BaseResponse getList(BaseParam param) {
        PageHelper.startPage(param.getPage(), param.getLimit());
        if (param.getEndDate() != null && !param.getEndDate().isEmpty())
            param.setEndDate(DateUtil.formatDate(DateUtil.addDays(DateUtil.parseDate(param.getEndDate()), 1)));
        List<UdplatHospital> list = mapper.list(param);
        PageInfo<UdplatHospital> pageInfo = new PageInfo<>(list);
        PageSimpleResponse<UdplatHospital> pageSimpleResponse = new PageSimpleResponse<>();
        pageSimpleResponse.setTotal(pageInfo.getTotal());
        pageSimpleResponse.setList(list);
        return ResultVOUtils.success(pageSimpleResponse);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseResponse save(List<UdplatHospitalModel> models, String type) {
        if (models != null && models.size() > 0) {
            for (UdplatHospitalModel item : models) {
                UdplatHospital entity = mapper.selectByPurchaseId(item.getPurchaseId());
                if (entity == null) {
                    entity = new UdplatHospital();
                    BeanUtils.copyProperties(item, entity);
                    entity.setCreateTime(new Date());
                    mapper.insert(entity);
                } else {
                    BeanUtils.copyProperties(item, entity);
                    entity.setUpdateTime(new Date());
                    mapper.updateByPrimaryKey(entity);
                }
            }
        }
        UdplatLog udplatLog = new UdplatLog();
        udplatLog.setDownloadType(type);
        udplatLog.setTbName("udplat_hospital");
        udplatLog.setType("info");
        udplatLog.setMsg("执行成功！");
        udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
        udplatLog.setTotalCount(models.size());
        udplatLog.setCreateTime(new Date());
        logService.insert(udplatLog);
        return ResultVOUtils.success();
    }

    @Override
    public void exportToFile(HttpServletResponse response) {
        List<UdplatHospital> list = mapper.selectAll();
        FileUtils.exportToFile(response, JSONObject.toJSONString(list), "医疗机构");
    }

}
