package com.glxp.udidl.admin.thread;

import com.glxp.udidl.admin.constant.Constant;
import com.glxp.udidl.admin.dao.info.ScheduledMapper;
import com.glxp.udidl.admin.entity.info.ScheduledEntity;
import com.glxp.udidl.admin.entity.udid.JobLog;
import com.glxp.udidl.admin.req.info.ScheduledRequest;
import com.glxp.udidl.admin.res.BaseResponse;
import com.glxp.udidl.admin.service.dataSync.ProductInfoDlService;
import com.glxp.udidl.admin.service.udi.JobLogService;
import com.glxp.udidl.admin.util.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 保存json数据到文件
 */
@Component
@EnableScheduling
public class DownloadProductInfoTask implements SchedulingConfigurer {

    final Logger logger = LoggerFactory.getLogger(DownloadProductInfoTask.class);

    @Autowired
    ProductInfoDlService productInfoDlService;
    @Resource
    private ScheduledMapper scheduledDao;
    @Autowired
    JobLogService jobLogService;
    @Resource
    UdiCompanyTask udiCompanyTask;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(() -> process(),
                triggerContext -> {
                    ScheduledRequest scheduledRequest = new ScheduledRequest();
                    scheduledRequest.setCronName("downloadProductInfo");
                    ScheduledEntity scheduledEntity = scheduledDao.findScheduled(scheduledRequest);
                    String cron = scheduledEntity.getCron();
                    if (cron.isEmpty()) {
                        logger.error("cron is null");
                    }
                    return new CronTrigger(cron).nextExecutionTime(triggerContext);
                });
    }

    private void process() {
        Date endDate = DateUtil.parseDate((DateUtil.formatDate(new Date())));
        Date startDate = DateUtil.addDays(endDate, -7);
        BaseResponse res = productInfoDlService.ExportToFile(startDate, endDate, Constant.DOWNLOAD_TYPE_AUTO);
        if (res.getCode() != 20000) {
            JobLog jobLog = new JobLog();
            jobLog.setType(Constant.LOG_TYPE_ERROR);
            jobLog.setDownloadType(Constant.DOWNLOAD_TYPE_AUTO);
            jobLog.setTaskType(Constant.JOB_TASK_TYPE_DOWNLOAD);
            jobLog.setMsg("下载文件失败:" + res.getMessage());
            jobLog.setCreateTime(new Date());
            jobLogService.insert(jobLog);
        }

        //更新一下企业信息
        udiCompanyTask.transAll();
    }
}
