package com.glxp.udidl.admin.thread;

import com.alibaba.fastjson.JSON;
import com.glxp.udidl.admin.dao.info.ScheduledMapper;
import com.glxp.udidl.admin.entity.info.ScheduledEntity;
import com.glxp.udidl.admin.entity.udplat.UdplatLog;
import com.glxp.udidl.admin.req.info.ScheduledRequest;
import com.glxp.udidl.admin.service.dataSync.UdplatSyncService;
import com.glxp.udidl.admin.service.udplat.UdplatLogService;
import com.glxp.udidl.admin.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

@Component
@EnableScheduling
@Slf4j
public class DownloadTaskUdplat implements SchedulingConfigurer {
    @Resource
    private ScheduledMapper scheduledDao;
    @Autowired
    UdplatSyncService udplatSyncService;
    @Autowired
    private UdplatLogService logService;            //日志

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.addTriggerTask(() -> process(),
                triggerContext -> {
                    ScheduledRequest scheduledRequest = new ScheduledRequest();
                    scheduledRequest.setCronName("downloadTaskUdplat");
                    ScheduledEntity scheduledEntity = scheduledDao.findScheduled(scheduledRequest);
                    String cron = scheduledEntity.getCron();
                    if (cron.isEmpty()) {
                        log.error("cron is null");
                    }
                    return new CronTrigger(cron).nextExecutionTime(triggerContext);
                });
    }

    private void process() {
        try {
            udplatSyncService.downloadByDay("auto");
        } catch (Exception e) {
            UdplatLog udplatLog = new UdplatLog();
            udplatLog.setDownloadType("auto");
            udplatLog.setType("error");
            udplatLog.setMsg(e.getMessage());
            udplatLog.setContent(JSON.toJSONString(e.getStackTrace()));
            udplatLog.setActionDate(DateUtil.parseDate(DateUtil.getDate()));
            udplatLog.setCreateTime(new Date());
            logService.insert(udplatLog);
        }
    }
}
